#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include "fbp_2021.h"
#include "feps_2011.h"

/* Emissions rates are based on CONSUME 3.0

Page numbers refer to:

    Consume 3.0 User's Guide
    Susan J. Prichard, Roger D. Ottmar, and Gary K. Anderson
    2004?

Bulk densities for FBP fuel types are based on

Anderson, K.R.  2000.  Incorporating smoldering into fire growth modelling.
    Pages 31-36 in 3rd Symp. on Fire and Forest Meteorology, Jan. 9-14, 2000,
    Long Beach, CA.  Am. Meteorol. Soc., Boston, MS.

*/

void EmissionsCalc(FEPS *feps, FBP *fbp, EMODELS *emodels, ALLOCATIONS *allocations)
{
double rho2, rho4, rho6, rho8, L, F, H, rhoL, rhoF, rhoH, mcL, mcF, mcH, LFC, FFC, HFC, DOB, DOB_F, DOB_S;
double Flaming, Smoldering, Residual;
double Flaming_SFC, Smoldering_SFC, Residual_SFC;   // added new variable Flaming_SFC -- 2020-10-23
double Flaming_GFC, Smoldering_GFC, Residual_GFC;   // added new variables -- 2022-11-15
double Flaming_CFC, Smoldering_CFC, Residual_CFC;   // added new variables -- 2022-11-15
double GFC, SFC, CFC, TFC;
int i, iFuel;

    Flaming = Smoldering = Residual = 0.;
    Flaming_SFC = Smoldering_SFC = Residual_SFC = 0.;
    Flaming_GFC = Smoldering_GFC = Residual_GFC = 0.;
    Flaming_CFC = Smoldering_CFC = Residual_CFC = 0.;
    LFC = FFC = HFC = 0.;

/* rename these for clarity */
    GFC = fbp->GFC;
    SFC = fbp->SFC-GFC; // keep SFC separate from GFC (included in the original FBP)
    TFC = fbp->TFC-GFC; // keep TFC separate from GFC (included in the original FBP)
    CFC = TFC-SFC;

    iFuel=-999;

    feps->Mw_flaming = feps->Mw_smoldering = feps->Mw_residual = 0.;

/* Default duff characteristics by fuel type */
    for (i=0; i<MAX_FUELS; i++) /* changed from 17 to MAX_FUELS - 2019/06/12 */
        if (!strncasecmp(fbp->FuelType, Fuels[i], 2))  iFuel= i;

    if (iFuel>=0 && iFuel<MAX_FUELS && feps->growth > 0. && fbp->TFC > 0.) /* changed from 17 to MAX_FUELS - 2019/06/12 */
    {


/* 1. Apply the average bulk density per fuel type to bulk densities for 2, 4 6 and 8 cm depths */

/* these values are in fbp_2009.h */
        rho2 = rhoBs[iFuel];
        rho4 = rhoBs[iFuel];
        rho6 = rhoBs[iFuel];
        rho8 = rhoBs[iFuel];

/* 2. Overwrite these values with depth specific values for certain fuels. */

        if (emodels->rho2[iFuel]>0.)  // values stored in emodels.csv
        {
            rho2 = emodels->rho2[iFuel];
            rho4 = emodels->rho4[iFuel];
            rho6 = emodels->rho6[iFuel];
            rho8 = emodels->rho8[iFuel];
        }
        else
        {

/* Use values from Anderson 2000 in g/cm^3 */
            if (!strncasecmp(fbp->FuelType, "C1", 2)) { rho2 = 0.045; rho4 = 0.045; rho6 = 0.045; rho8 = 0.045; }
            if (!strncasecmp(fbp->FuelType, "C2", 2)) { rho2 = 0.019; rho4 = 0.034; rho6 = 0.051; rho8 = 0.056; }
            if (!strncasecmp(fbp->FuelType, "C3", 2)) { rho2 = 0.015; rho4 = 0.020; rho6 = 0.032; rho8 = 0.066; }
            if (!strncasecmp(fbp->FuelType, "C4", 2)) { rho2 = 0.022; rho4 = 0.029; rho6 = 0.045; rho8 = 0.059; }
            if (!strncasecmp(fbp->FuelType, "C5", 2)) { rho2 = 0.093; rho4 = 0.093; rho6 = 0.093; rho8 = 0.093; }
            if (!strncasecmp(fbp->FuelType, "C6", 2)) { rho2 = 0.030; rho4 = 0.050; rho6 = 0.050; rho8 = 0.050; }
            if (!strncasecmp(fbp->FuelType, "C7", 2)) { rho2 = 0.100; rho4 = 0.100; rho6 = 0.050; rho8 = 0.050; } /* values used to match SFC limits in FBP */

            if (!strncasecmp(fbp->FuelType, "D1", 2)) { rho2 = 0.061; rho4 = 0.061; rho6 = 0.061; rho8 = 0.061; } /* in the CFFEPS documentation */

/* not sure where these came from but they are in the CCFFEPS documentation */
            if (!strncasecmp(fbp->FuelType, "M1", 2)) { rho2 = (0.019+0.034)/2; rho4 = (0.034+0.108)/2; rho6 = (0.051+0.108)/2; rho8 = (0.056+0.108)/2; } /* average of C2 and D1*/
            if (!strncasecmp(fbp->FuelType, "M2", 2)) { rho2 = (0.019+0.034)/2; rho4 = (0.034+0.108)/2; rho6 = (0.051+0.108)/2; rho8 = (0.056+0.108)/2; } /* average of C2 and D1*/

            if (!strncasecmp(fbp->FuelType, "M3", 2)) { rho2 = 0.041; rho4 = 0.061; rho6 = 0.084; rho8 = 0.112; }
            if (!strncasecmp(fbp->FuelType, "M4", 2)) { rho2 = 0.041; rho4 = 0.061; rho6 = 0.084; rho8 = 0.112; }

            if (!strncasecmp(fbp->FuelType, "S1", 2)) { rho2 = 0.2; rho4 = 0.2; rho6 = 0.2; rho8 = 0.2; } /* values used to match SFC limits in FBP */
            if (!strncasecmp(fbp->FuelType, "S2", 2)) { rho2 = 0.5; rho4 = 0.5; rho6 = 0.3; rho8 = 0.3; } /* values used to match SFC limits in FBP */
            if (!strncasecmp(fbp->FuelType, "S3", 2)) { rho2 = 0.6; rho4 = 0.6; rho6 = 1.0; rho8 = 1.0; } /* values used to match SFC limits in FBP */
        }


/* old numbers */
        L = 2.;
        F = 5. - L;
        H = 25 - F - L;
        rhoL = rho2;
        rhoF = (2*rho4+rho6)/3.;
        rhoH = (rho6+2*rho8)/3.;

/* 3. Calculate the bulk densities of the L (0-1.2 cm), F (1.2-7 cm) and H (7-18 cm) layers from the 2 cm depth data. */

/* Nominal Fuel Depths [cm] from Van Wagner 1987, Table 1. */
        L = 1.2;
        F = 7. - L;
        H = 18. - F - L;

        rhoL = rho2;
        rhoF = ((2.-L)*rho2 + 2*rho4 + 2*rho6 + rho8)/F;
        rhoH = rho8;

// Standard moisture content equations (and convert from percent to fraction) -- now required for the subroutine PlumeCalcWet -- 2019-07-17
        mcL=mcF=mcH=0.;
        if (fbp->FFMC > 0.)
            mcL = 147.2*(101.-fbp->FFMC)/(59.5+fbp->FFMC) / 100.;
        if (feps->DMC > 0.)
            mcF = (exp((feps->DMC - 244.72)/(-43.43))+20.) / 100.;

        if (feps->DC > 0.)
//            mcH = 800./exp(feps->DC/400.) / 100.;      // note: this is actually moisture equivalent in hundreths of an inch!
            mcH = 400./exp(feps->DC/400.) / 100.;      // Dan Thompson's equation based on BMW's assumption that DC is saturated at 400% -- 2019-07-17




/* 3.a. If grass fuel type (O1), burn off the entire grass fuel load. Use the grass reduction values for flaming, smoldering and residual combustion (see Table 2). * /
        if (!strncasecmp(fbp->FuelType, "O1", 2)) /* These values are based on a personal communication with Bill de Groot */

        if (GFC > 0.)
        {
            Flaming_GFC = allocations->GFC[0]* GFC;
            Smoldering_GFC = allocations->GFC[1]* GFC;
            Residual_GFC = allocations->GFC[2]* GFC;

/* Water released [kg/m^2] released due to fuel moisture (mcL, mcF, mcH), required for the subroutine PlumeCalcWet -- 2019-07-17*/
            if (mcL>0.)
            {
                feps->Mw_flaming = Flaming_GFC*mcL;
                feps->Mw_smoldering = Smoldering_GFC*mcL;
                feps->Mw_residual = Residual_GFC*mcL;
            }

            DOB = DOB_F = DOB_S = 1.;    /* previously not calculated, not needed but now set to 1 cm DOB so the gras has a chance to smolder - 2023/05/10*/
            LFC = FFC = HFC = 0.;    /* not calculated, not needed */
        }

        if (TFC > 0.)
        {

/* 3.b. else, if slash, use the total fuel consumption (TFC). Use the slash reduction values for flaming, smoldering and residual combustion (see Table 2). */

/* Slash  - p 194*/
            if (!strncasecmp(fbp->FuelType, "S", 1))
            {
                Flaming_SFC = (allocations->FFC[0]*fbp->FFL + allocations->WFC[0]*fbp->WFL)/(fbp->FFL + fbp->WFL) * TFC;
                Smoldering_SFC = (allocations->FFC[1]*fbp->FFL + allocations->WFC[1]*fbp->WFL)/(fbp->FFL + fbp->WFL) * TFC;
                Residual_SFC = (allocations->FFC[2]*fbp->FFL + allocations->WFC[2]*fbp->WFL)/(fbp->FFL + fbp->WFL) * TFC;

/* Water released [kg/m^2] due to fuel moisture (mcL, mcF, mcH), required for the subroutine PlumeCalcWet -- 2019-07-17*/
                if (mcL>0. && mcF>0. && mcH>0.&& (fbp->FFL + fbp->WFL) > 0.) // [kg/m2] - calculate only when mcL, mcF, and mcF are all > 0. else 0.
                {
/* this section seems to be overwritten -- 2022-11-15
                    feps->Mw_flaming    = 0.70* TFC*mcL;
                    feps->Mw_smoldering = 0.15* TFC*mcF;
                    feps->Mw_residual   = 0.15* TFC*mcH; // was mcH  */

// Assumption that mcL alone is used for flaming, etc., (even fro WFL) to keep the equations consistent
                    feps->Mw_flaming += (allocations->FFC[0]*fbp->FFL + allocations->WFC[0]*fbp->WFL) * mcL / (fbp->FFL + fbp->WFL) *  TFC ;
                    feps->Mw_smoldering += (allocations->FFC[1]*fbp->FFL + allocations->WFC[1]*fbp->WFL) * mcF / (fbp->FFL + fbp->WFL) *  TFC;
                    feps->Mw_residual += (allocations->FFC[2]*fbp->FFL + allocations->WFC[2]*fbp->WFL) * mcH / (fbp->FFL + fbp->WFL) *  TFC;
// += used to accumulate moisture released by all approaches -- 2022-11-15

//printf("%s %lf %lf %lf\n", fbp->FuelType, feps->Mw_flaming, feps->Mw_smoldering, feps->Mw_residual);
                }

            }

/* Tropical Peat */
            if (!strncasecmp(fbp->FuelType, "O", 1))
            {
                rhoL = .1;
                rhoF = .1;
                rhoH = .1;

                Flaming = 0.;
                Smoldering = 0.5 * TFC;
                Residual = 0.5 * TFC;

/* Water released [kg/m^2] due to fuel moisture (mcL, mcF, mcH), required for the subroutine PlumeCalcWet -- 2019-07-17 (updated 2022-11-15) */
                if (mcF>0. && mcH>0.)
                {
                    feps->Mw_flaming    += 0.;
                    feps->Mw_smoldering += Smoldering*mcF;
                    feps->Mw_residual   += Residual*mcH;
                }   // += used to accumulate moisture released by all approaches -- 2022-11-15

            }
            else
            {

/* 3.c.    else, burn off the LFH layers in sequence until the entire SFC is accounted for.  Then use the crown fuel consumption (CFC) for any canopy burned.  Use the corresponding reduction values for flaming, smoldering and residual combustion (see Table 2). */

                if ( SFC < L*rhoL*10.) /* rhoL*10 give mass per square metre per cm of depth; rho*20. gives mass per 2cm layer */
                    LFC =  SFC;
                else
                    LFC = L*rhoL*10.;

                if ( SFC-LFC < F*rhoF*10.) /* rho*10 give mass per square metre per cm of depth; rho*20. gives mass per 2cm layer */
                    FFC =  SFC-LFC;
                else
                    FFC = F*rhoF*10.;

                if (FFC < 0.)
                    FFC = 0.;

                HFC =  SFC-LFC-FFC;

                if (HFC < 0.)
                    HFC = 0.;

/* Ground Fuels [kg/m^2] - p. 166 */
                Flaming_SFC    = allocations->SFC_L[0]*LFC + allocations->SFC_F[0]*FFC + allocations->SFC_H[0]*HFC;   // added new variable Flaming_SFC -- 2020-10-23
                Smoldering_SFC = allocations->SFC_L[1]*LFC + allocations->SFC_F[1]*FFC + allocations->SFC_H[1]*HFC;
                Residual_SFC   = allocations->SFC_L[2]*LFC + allocations->SFC_F[2]*FFC + allocations->SFC_H[2]*HFC;

/* Trees (CFC) - p. 169 */
                Flaming_CFC    = allocations->CFC_F[0]*CFC;   /* using mid story as compromise between over/mid and understory */
                Smoldering_CFC = allocations->CFC_F[1]*CFC;  /* " / 0.85 " used to equate Flaming + Smouldering to CFC */
                Residual_CFC   = allocations->CFC_F[2]*CFC;  /* " / 0.85 " used to equate Flaming + Smouldering to CFC */

/* Water released [kg/m^2] due to fuel moisture (mcL, mcF, mcH), required for the subroutine PlumeCalcWet -- 2019-07-17 (updated 2022-11-15) */
                if (mcL>0. && mcF>0. && mcH>0.) // calculate only when mcL, mcF, and mcF are all > 0. else 0.
                {   // corrected to include FMC 2020-10-13
                    feps->Mw_flaming     += allocations->SFC_L[0]*LFC*mcL + allocations->SFC_F[0]*FFC*mcF + allocations->SFC_H[0]*HFC*mcH + allocations->CFC_F[0]*fbp->FMC/100.;
                    feps->Mw_smoldering  += allocations->SFC_L[1]*LFC*mcL + allocations->SFC_F[1]*FFC*mcF + allocations->SFC_H[1]*HFC*mcH + allocations->CFC_F[1]*fbp->FMC/100.;
                    feps->Mw_residual    += allocations->SFC_L[2]*LFC*mcL + allocations->SFC_F[2]*FFC*mcF + allocations->SFC_H[2]*HFC*mcH + allocations->CFC_F[2]*fbp->FMC/100.;
                }   // += used to accumulate moisture released by all approaches -- 2022-11-15

            }

/* 4. Calculate depth of burn [cm] per stage */
            DOB =  SFC/(rhoL*10.);
            if (DOB > L) DOB = L + ( SFC - L*rhoL*10.)/(rhoF*10.);
            if (DOB > L+F)  DOB = L + F + ( SFC - L*rhoL*10. - F*rhoF*10.)/(rhoH*10.);

/* not quite but close enough    // added new variable Flaming_SFC -- 2020-10-23 */
            DOB_F = Flaming_SFC/(rhoL*10);
            if (DOB_F > L) DOB_F = L + (Flaming_SFC - L*rhoL*10.)/(rhoF*10.);
            if (DOB_F > L+F)  DOB_F = L + F + (Flaming_SFC - L*rhoL*10. - F*rhoF*10.)/(rhoH*10.);

            DOB_S = DOB-DOB_F;

        }

//        printf("... %lf %lf\n", Flaming, Flaming_SFC);

// Flaming/Smoldering/Residual amounts in kg/m^2
        feps->Flaming = Flaming_GFC+Flaming_SFC+Flaming_CFC;
        feps->Smoldering = Smoldering_GFC+Smoldering_SFC+Smoldering_CFC;
        feps->Residual = Residual_GFC+Residual_SFC+Residual_CFC;

// Add in moisture released due to chemical decomposition -- 2022-11-15 (was included in previous calculations)
// 0.60 = mass water produced during dry combustion (C6H1206 + 6O2 -> 6 CO2 + 6H20)  pg 157, Brown and Davis 2nd ed 1973
        feps->Mw_flaming     += 0.60*feps->Flaming;
        feps->Mw_smoldering  += 0.60*feps->Smoldering;
        feps->Mw_residual    += 0.60*feps->Residual;
// += used to accumulate moisture released by all approaches -- 2022-11-15

        feps->DOB = DOB;
        feps->DOB_F = DOB_F;
        feps->DOB_S = DOB_S;

//        printf ("%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf %lf\n", feps->DMC, mcL, mcF, mcH, LFC, FFC, HFC, Flaming, Smoldering, Residual, feps->Mw_flaming , feps->Mw_smoldering, feps->Mw_residual, feps->Mw_flaming+feps->Mw_smoldering+feps->Mw_residual);

/* 5. Calculate energy release per stage */
        feps->Qflaming = feps->Qplume * feps->Flaming /  fbp->TFC;
        feps->Qsmoldering = feps->Qplume * feps->Smoldering /  fbp->TFC;
        feps->Qresidual = feps->Qplume * feps->Residual /  fbp->TFC;

    }
    else
    {

        feps->Flaming = 0.;
        feps->Smoldering = 0.;
        feps->Residual = 0.;

        feps->Mw_flaming = 0.;
        feps->Mw_smoldering = 0.;
        feps->Mw_residual = 0.;

        feps->DOB = 0.;
        feps->DOB_F = 0.;
        feps->DOB_S = 0.;

        feps->Qflaming = 0.;
        feps->Qsmoldering = 0.;
        feps->Qresidual = 0.;

    }
}

/* new code used in CFFEPS.c - incorporates the residence time per stage */

/*  In this approach, the residence times in each of the three are passed as fractions of an hour through the feps structure.
    It assumes that each phase is completed before the next begins
    It also assumes that the emissions rate in each phase is constant during the phase */

int EmissionsOverTime(double *Fs, double *Ss, double *Rs, int jModel, int iSpecies, EMISSIONS *emissions, FEPS *feps )
{
   double Fr, Sr, Rr, Ft, St, Rt, Fp, Sp, Rp, n, a, b, c, t1, t2;
   int i, imax, ntimesteps;

/* 1. Calculate the total residence time in hours for flaming, smoldering and residual stages */
//     recalculated from hours to timesteps 2018-07-30
    imax = (int)ceil( (emissions->residence_flaming + emissions->residence_smoldering + emissions->residence_residual) / feps->timestep);
    ntimesteps = (int)(MAX_HOURS/feps->timestep)+1;
    if (imax > ntimesteps) imax = ntimesteps;   // to be consistent with CFFEPS.c - KRA 2021-03-12
//    if (imax > MAX_TIMESTEPS) imax = MAX_TIMESTEPS;
//    if (jModel==-1) printf("imax = %d %lf %lf %lf %lf\n", imax, feps->timestep, emissions->residence_flaming, emissions->residence_smoldering, emissions->residence_residual);

    Fr = Sr = Rr = 0.;

    for (i=0; i<imax;i++)
        Fs[i] = Ss[i] = Rs[i] = 0.; // deletes any lingering values -- KRA -2020-09-16

    if (jModel >= 0)
    {
        a = emissions->a[jModel][iSpecies];
        b = emissions->b[jModel][iSpecies];
        c = emissions->c[jModel][iSpecies];

/* 2. Calculate the emissions rates [tonnes/hr] (or energy if jModel <0) for flaming, smoldering and residual stages */

//   tonnes/hr =             ha/timestep  / hours/timestep *      tonnes/ha
        if (emissions->residence_flaming > 0.)
            Fr = a / 1000. * feps->growth / feps->timestep * 10.*feps->Flaming;// / emissions->residence_flaming;   // was a/2000 for [lbs/ton] - KRA 2017-02-06
        else
            Fr =0.;

        if (emissions->residence_smoldering > 0.)
            Sr = b / 1000. * feps->growth / feps->timestep  * 10.*feps->Smoldering;// / emissions->residence_smoldering; // was b/2000
        else
            Sr = 0.;

        if (emissions->residence_residual > 0.)
            Rr = c / 1000. * feps->growth / feps->timestep  * 10.*feps->Residual;// / emissions->residence_residual; // was c/2000
        else
            Rr = 0.;

/* emissions factors a, b and c are now assumed to be in [g/kg] a / 1000. * (not [lbs/ton] as described below) - KRA 2017-02-06 */

/*                 [lbs/ton]     [lbs/ton]     [ha]         [kg/m^2]         [hours]

   a, b and c are in lbs/ton    (changed to [g/kg] - 2017-02-06)
   divide by 2000 to convert lbs/ton to unitless (cancels units of a, b, c)
   feps->growth is in hectares
   feps->Flaming is a fraction of the TFC in kg/m^2 (coverted to tonnes per hectare by multiplying by 10)
   emissions->residence_flaming[iSpecies]]

   answer is in [metric] tonnes per hour
*/
    }
    else if (jModel == -1)    /* 2a. if jModel == -1, use this technique to come up with energy per hour */
    {
//               J/timesteps    / hrs/timestep
        if (emissions->residence_flaming > 0.)
            Fr = feps->Qflaming / feps->timestep;// / emissions->residence_flaming;
        else
            Fr =0.;

        if (emissions->residence_smoldering > 0.)
            Sr = feps->Qsmoldering / feps->timestep;// / emissions->residence_smoldering;
        else
            Sr = 0.;

        if (emissions->residence_residual > 0.)
            Rr = feps->Qresidual / feps->timestep;// / emissions->residence_residual;
        else
            Rr = 0.;
    }
    else if (jModel == -2)    /* 2b. if jModel == -2, use this technique to come up with water released per hour, required for the subroutine PlumeCalcWet -- 2019-07-17*/
    {
    //   kg/hr =           m^2/timestep  / hours/timestep * kg/m^2
        if (emissions->residence_flaming > 0. && feps->Mw_flaming > 0.)
            Fr = 10000.*feps->growth / feps->timestep * feps->Mw_flaming;
        else
            Fr =0.;

        if (emissions->residence_smoldering > 0. && feps->Mw_smoldering > 0.)
            Sr = 10000.*feps->growth / feps->timestep  * feps->Mw_smoldering;
        else
            Sr = 0.;

        if (emissions->residence_residual > 0. && feps->Mw_residual > 0.)
            Rr = 10000.*feps->growth / feps->timestep  * feps->Mw_residual;
        else
            Rr = 0.;
    }
    else    /* 2c. if jModel < -2, use this technique to come up with fuel consumed per hour -- 2019-07-25*/
    {
//   tonnes/hr = ha/timestep  / hours/timestep    tonnes/ha
        if (emissions->residence_flaming > 0.)
            Fr = feps->growth / feps->timestep  * 10*feps->Flaming;
        else
            Fr =0.;

        if (emissions->residence_smoldering > 0.)
            Sr = feps->growth / feps->timestep  * 10*feps->Smoldering;
        else
            Sr = 0.;

        if (emissions->residence_residual > 0.)
            Rr = feps->growth / feps->timestep  * 10*feps->Residual;
        else
            Rr = 0.;
    }

/* Resident times (hours) */
    Ft = emissions->residence_flaming;
    St = emissions->residence_smoldering;
    Rt = emissions->residence_residual;
// printf("Ft: %lf St: %lf Rt:%lf\n", Ft, Rt, St);

/* Partial Resident times (hours) */
    Fp = Sp = Rp = 0.;

/* 3. Stepping through each timestep */
    if (Fr+Sr+Rr > 0)
        for (i=0; i<imax;i++)
        {
            t1 = i * feps->timestep;
            t2 = t1 + feps->timestep;
/* 3.a. calculate partial residence times remaining in decimal hours * /
            if (t2 <= Ft)
                Fp = feps->timestep;
            else
                if (t1 < Ft)
                    Fp = modf(Ft-t1, &n);
                else
                    Fp = 0.;

            if (t2 <= Ft + St)
                Sp = feps->timestep - Fp;
            else
                if (t1 < Ft + St)
                    Sp = modf(Ft+St-t1, &n) - Fp;
                else
                    Sp = 0.;

            if (t2 <= Ft + St + Rt)
                Rp = feps->timestep - Sp - Fp;
            else
                if (t1 < Ft + St + Rt)
                    Rp = modf(Ft+St+Rt-t1, &n) - Sp - Fp;
                else
                    Rp = 0.;*/

/* 3.a. calculate partial residence times remaining in decimal hours -- rewritten to correctly handle variable timesteps 2021-04-13 */
            if (Ft <= t1)
                Fp = 0.;
            else if (Ft >= t2)
                    Fp = t2-t1;
                else
                    Fp = (Ft)-t1;

            if (Ft+St <= t1)
                Sp = 0.;
            else if (Ft+St >= t2)
                    Sp = t2-t1 - Fp;
                else
                    Sp = (Ft+St)-t1 - Fp;

            if (Ft+St+Rt <= t1)
                Rp = 0.;
            else if (Ft+St+Rt >= t2)
                    Rp = t2-t1 - Fp - Sp;
                else
                    Rp = (Ft+St+Rt)-t1 - Fp - Sp;

// printf("t1: %lf t2: %lf... Fp: %lf Sp: %lf Rp: %lf\n", t1, t2, Fp, Sp, Rp);

/* 3.b. calculate the emissions for the partial hour */
//     recalculated from hours to timesteps 2018-07-30

// tonnes/timestep = tonnes/hr * hrs * hrs/timestep / hrs
            if (emissions->residence_flaming > 0.)
                Fs[i] = Fr*Fp * feps->timestep / emissions->residence_flaming;
            else
                Fs[i] = 0;

            if(emissions->residence_smoldering > 0.)
                Ss[i] = Sr*Sp * feps->timestep / emissions->residence_smoldering;
            else
                Ss[i] = 0.;

            if (emissions->residence_residual > 0.)
                Rs[i] = Rr*Rp * feps->timestep / emissions->residence_residual;
            else
                Rs[i] = 0.;

//printf("i: %d Fs[i]:%lf Ss[i]: %lf Rs[i]:%lf\n", i, Fs[i], Ss[i], Rs[i]);

        }

    return(imax);

}

double Proportion (double x1, double x2, double y1, double y2)
{
double P;
// calculations the proportion of variable (x1<x2) within (y1<y2)

    if (x2 > y2)
    {
        if (x1 > y2)
            P=0.;                       // (x1<x2) range > (y1<y2)
        else
            if (x1 > y1)
                P = (y2-x1)/(x2-x1);    // (x1<x2) straddles (y2)
            else
                P = (y2-y1)/(x2-x1);    // (y1<y2) within (x1<x2)
    } // x2 <= y2
    else
        if (x2 < y1)
            P=0.;                       // (x1<x2)  < (y1<y2)
        else
            if (x1 < y1)
                P = (x2-y1)/(x2-x1);    // (x1<x2) straddles (y1)
            else
                P = 1.;                 // (x1<x2) within (y1<y2)

// printf("... %lf %lf %lf %lf %lf\n", P, x1, x2, y1, y2);
    return (P);
}
/*  In this approach, the concentration downwind (y) is calculated, integrated for the entire cross-section of the plume.
    time since ignition is passed as dt
    It assumes that each phase is completed before the next begins
    It also assumes that the emissions rate in each phase is constant during the phase */

double EmissionsOverDistance(double *Cy, double Mdd, double dy, int jModel, int iSpecies, EMISSIONS *emissions, FEPS *feps, FBP *fbp )
{
double Ft, St, Rt, Fc, Sc, Rc, Md, Fd, Sd, Rd, Zd, Fdd, Sdd, Rdd, Zdd, y, a, b, c, x1, x2, y1, y2, maxy;
int i;


// printf("%s %lf %lf %lf %lf %lf %lf %lf ", fbp->FuelType, feps->elapsed, feps->Flaming, feps->Smoldering, feps->Residual, emissions->residence_flaming, emissions->residence_smoldering, emissions->residence_residual);

// printf("%lf %lf %lf ", feps->elapsed, fbp->WS, Mdd);

/* emissions factors a, b and c in [g/kg] for flaming, smoldering and residual stages */

    a = emissions->a[jModel][iSpecies];
    b = emissions->b[jModel][iSpecies];
    c = emissions->c[jModel][iSpecies];

/* Resident times (hours) */
    Ft = emissions->residence_flaming;
    St = emissions->residence_smoldering;
    Rt = emissions->residence_residual;

// Depths of plumes (km)
    Fd = fbp->WS * Ft;  // depth of flame's plume
    Sd = fbp->WS * St;  // depth of smoldering's plume
    Rd = fbp->WS * Rt;  // depth of residual's plume

    if (Rd < 0.) Rd = 0.;
    if (Sd < 0.) Sd = 0.;
    if (Fd < 0.) Fd = 0.;

// downwind distances (km)
    if (Mdd<=0.)
        Mdd = fbp->WS * feps->elapsed;  // Max downwind distance
    else
        Mdd = Mdd + fbp->WS * feps->timestep;  // Max downwind distance

    Zdd = Mdd - Fd - Sd - Rd; // distance to tail of residual's plume

    Rdd =  Zdd + Rd;
    Sdd =  Rdd + Sd;
    Fdd =  Sdd + Fd;

    if (Zdd < 0.) Zdd = 0.;
    if (Mdd < 0.) Mdd = 0.;

// downwind concentration (g/km) or (kg/km)
    Fc = Sc = Rc = 0.;

// [kg/km] or [g/m] = [g/kg]/[g/kg] * [kg/m^2] / [km]
    if (Fd > 0.) Fc = a / 1000. * feps->Flaming / Fd;
    if (Sd > 0.) Sc = b / 1000. * feps->Smoldering / Sd;
    if (Rd > 0.) Rc = c / 1000. * feps->Residual / Rd;

// printf("%lf %lf %lf %lf %lf ", Zdd, Rdd, Sdd, Fdd, Mdd);

// printf ("%lf %lf %lf %lf %lf %lf", Rc, Rd, Sc, Sd, Fc, Fd);
    for (i = 0; i < 50; i++)
    {
        y1= i*dy; y2 = y1+dy;

// accumulated concentrations downwind
//   [tonnes/bin] = [kg/km] * [km] * [fraction/bin] / [kg/tonnes]
            Cy[i] = Rc * Rd * Proportion (Zdd, Rdd, y1, y2) / 1000.;            // residual
            Cy[i] = Cy[i] + Sc * Sd * Proportion (Rdd, Sdd, y1, y2) / 1000.;    // smoldering
            Cy[i] = Cy[i] + Fc * Fc * Proportion (Sdd, Fdd, y1, y2) / 1000.;    // flaming

// printf(" %lf", Cy[i]);
    }
// printf("\n");

    return(Mdd);

}
