!============================================================================!
!
! Projet / Project : GEM-MACH
! Fichier / File   : fbp_2019.f90
! Creation         : Kerry Anderson - June 5, 2019
!                  : (C To Fortran)
!                    A. Akingunola, J. Chen, and P. Makar - Fall 2018
! Description      : Fire Behavior Prediction System for CFFEPS
! This subroutine represents equations from
!
! Wotton, B.M.; Alexander, M.E.; Taylor, S.W.  2009,.  Updates and revisions to the
!    1992 Canadian Forest Fire Behavior Prediction System .  Natural Resources
!    Canada, Canadian Forest Service, Great Lakes Forestry Centre, Sault Ste. Marie,
!    Ontario, Canada.  Information Report GLC-X-10, 45 p.
!
! Updates are indicated with a "- 2009" in the equation number comment on the right
!
! Updated to include various SFC calculation method (2020-02-21)
!
!   Discrepancies between BMW's fbp.c and ST-X-3
!
!   eqn 32: BMW has 33.5 instead of 35.5 (see ROScalc and Slopecalc)
!   eqn 57: BMW had forced SFC to 2.0 for C6 (this is now gone)
!   b[O1b]: BMW has 0.0829 instead of 0.0310
!   c[O1a]: BMW has 1.41 instead of 1.4
!
!
! Note: There is no attempt to account for greened-up deciduous (D2)
!============================================================================
!
! **********************************************

module FBP_module

   use CFFEPS_module

   implicit none
   private
   public  :: FBPCalc_f90, FBPtest

   real(kind=4), parameter :: PI = 3.1415926, NoBUI = -1.0

! Fuel numbers matched with iFuel
   integer (kind=4), parameter :: C1 = 1, C2 = 2, C3 = 3, C4 = 4, &
                                  C5 = 5, C6 = 6, C7 = 7, &
                                  D1 = 8, D2 = 9, &
                                  M1 = 10, M2 = 11, M3 = 12, M4 = 13, &
                                  S1 = 14, S2 = 15, S3 = 16, &
                                  O1a = 17, O1b = 18, WA = 19, NF = 20, &
                                  nofuel = -1

   real(kind=4), dimension(MAX_FUELS), parameter :: a = &
                    (/  90.,  110.,   110.,  110.,   30.,   30., &
                        45.,   30.,   30.,     0.,    0.,  120., &
                       100.,   75.,   40.,   55.,  190.,   250., &
                         0.,   0. /)
   real(kind=4), dimension(MAX_FUELS), parameter :: b = &
                    (/.0649, .0282,  .0444, .0293, .0697, .0800, &
                      .0305, .0232, .0232,     0.,    0., .0572, &
                      .0404, .0297, .0438, .0829, .0310,  .0350, &
                         0.,   0.  /)
   real(kind=4), dimension(MAX_FUELS), parameter :: c = &
                    (/  4.5,   1.5,    3.0,   1.5,   4.0,   3.0, &
                        2.0,   1.6,    1.6,    0.,    0.,   1.4, &
                       1.48,   1.3,    1.7,   3.2,   1.4,   1.7, &
                         0.,   0. /)
   real(kind=4), dimension(MAX_FUELS), parameter :: q = &
                    (/ 0.90,  0.70,   0.75,  0.80,  0.80,  0.80, &
                       0.85,  0.90,   0.90,    .8,    .8,    .8, &
                         .8,  0.75,   0.75,  0.75,  1.00,  1.00, &
                         0.,   0.  /)
   real(kind=4), dimension(MAX_FUELS), parameter :: BUIo = &
                    (/ 72.,   64.,   62.,    66.,   56.,   62.,  &
                      106.,   32.,   32.,    50.,   50.,   50.,  &
                       50.,   38.,   63.,    31.,    1.,    1.,  &
                        0.,    0./)
   real(kind=4), dimension(MAX_FUELS), parameter :: CBHs = &
                    (/  2.,    3.,     8.,    4.,   18.,    7.,   &
                       10.,    0.,     0.,    6.,    6.,    6.,   &
                        6.,    0.,     0.,    0.,    0.,    0.,   &
                        0.,    0./)
   real(kind=4), dimension(MAX_FUELS), parameter :: CFLs = &
                    (/ .75,   .80,   1.15,  1.20,  1.20,  1.80,   &
                       .50,    0.,     0.,   0.8,   0.8,   0.8,   &
                       0.8,    0.,     0.,    0.,    0.,    0.,   &
                        0.,    0./)

! These values are pulled from the SFC calculations -- updated 2020-02-23
   real(kind=4), dimension(MAX_FUELS), parameter :: SFLs = &
                    (/ 1.5,   5.0,    5.0,   5.0,   5.0,   5.0,   &
                        0.,   1.5,    1.5,  3.25,  3.25,   5.0,   &
                       5.0,    0.,     0.,    0.,    0.,    0.,   &
                        0.,    0./)

   real(kind=4), dimension(MAX_FUELS), parameter :: FFLs = &
                    (/  0.,    0.,     0.,    0.,    0.,    0.,   &
                       2.0,    0.,     0.,    0.,    0.,    0.,   &
                        0.,   4.0,  10.0,  12.0,     0.,    0.,   &
                        0.,   0./)
   real(kind=4), dimension(MAX_FUELS), parameter :: WFLs = &
                    (/  0.,    0.,     0.,    0.,    0.,    0.,   &
                       1.5,    0.,     0.,    0.,    0.,    0.,   &
                        0.,   4.0,   6.0,  20.0,     0.,    0.,   &
                        0.,   0./)

! These values taken from Anderson 2002
   real(kind=4), dimension(MAX_FUELS), parameter :: depths = &
                    (/  3.4,  10.0,   6.5,   4.6,   5.0,   5.0,  &
                        5.0,   2.4,   2.4,   5.0,   5.0,   7.5,   &
                        7.5,   7.4,   7.4,   7.4,    0.,    0.,  &
                        0.,   0./)

   real(kind=4), dimension(MAX_FUELS), parameter :: rhoBs = &
                    (/ 0.045, 0.034,   0.026, 0.028, 0.093, 0.05,   &
                       0.02,  0.108,   0.108, 0.061, 0.061, 0.106,  &
                       0.106, 0.078, 0.132,   0.1,      0.,     0., &
                       0.,   0. /)

   real(kind=4), dimension(MAX_FUELS), parameter :: Ris = &
                    (/ 0.05,   0.0,  0.15,  0.15,  0.15,  0.15,  &
                       0.15,  0.59,  0.59,  0.25,  0.25,  0.15,   &
                       0.15,  0.15,  0.15,  0.15,    0.,    0.,  &
                        0.,   0./)


   character(len=FUEL_SIZE), dimension(MAX_FUELS), parameter :: &
                    Fuels = &
                    (/ "C1  ", "C2  ", "C3  ", "C4  ", "C5  ", "C6  ", &
                       "C7  ", "D1  ", "D2  ", "M1  ", "M2  ", "M3  ", &
                       "M4  ", "S1  ", "S2  ", "S3  ", "O1a ", "O1b ", &
                       "WA  ", "NF  "/)

   contains

   integer(kind=4) function FBPCalc_f90(fbp) result(iStatus)
      implicit none
      type(fbp_type),  intent(inout) :: fbp

      character(len=FUEL_SIZE) :: FuelType

      integer(kind=4) :: Accel, Dj, D0, ELV, BUIEff, method

      real(kind=4) :: t, FFMC, ISI, BUI, WS, WD, GS, Aspect, PC, PDF, GC, &
                      DMC, DC, Depth, BD, &
                      GFL, CBH, CFL, LAT, LON, FMC, SH, SD, theta, &
                      ROS, HROSt, CFB, HFI, TFC, TI, &
                      FROS, FROSt, FCFB, FFI, FTFC, FTI, &
                      BROS, BROSt, BCFB, BFI, BTFC, BTI, &
                      TROS, TROSt, TCFB, TFI, TTFC, TTI, &
                      SFC, LB, RAZ, WSV, &
                      SFL, FFL, WFL, &
                      E, LBt, ROSt, RSC, SAZ, WAZ, &
                      SFCmax

      integer(kind=4) :: iFuel

      iStatus = 0    ! success

      FuelType = fbp%FuelType

      Accel  = fbp%Accel                ! 0 = on, else off
      Dj     = fbp%Dj
      D0     = fbp%D0
      ELV    = fbp%ELV
      BUIEff = fbp%BUIEff               ! 0 = on, else off
      method = fbp%FC_method            ! default = 1  for standard FBP SFC calculations
      t      = fbp%t
      FFMC   = fbp%FFMC
      DMC    = fbp%DMC                  ! added 2019-11-01
      DC     = fbp%DC                   ! added 2019-11-01
      Depth  = fbp%Depth                ! added 2019-11-01
      BD     = fbp%BD                   ! added 2019-11-01
      ISI    = fbp%ISI
      BUI    = fbp%BUI
      WS     = fbp%WS
      WD     = fbp%WD * PI/180.         ! radians
      GS     = fbp%GS
      Aspect = fbp%Aspect * PI/180.     ! radians
      PC     = fbp%PC
      PDF    = fbp%PDF
      GC     = fbp%C                    ! Grass Curing
      GFL    = fbp%GFL
      CBH    = fbp%CBH
      CFL    = fbp%CFL
      SFL    = fbp%SFL                  ! added 2019-07-25
      FFL    = fbp%FFL                  ! added 2019-07-25
      WFL    = fbp%WFL                  ! added 2019-07-25
      LAT    = fbp%LAT
      LON    = fbp%LON
      FMC    = fbp%FMC
      theta  = fbp%theta * PI/180.      ! radians
      SD     = fbp%SD
      SH     = fbp%SH

!  Some default values
      if (Accel <= 0) then
        Accel = 0
      else
        Accel = 1 ! line=1 (no accelleration effect) -- changed from < 0 to <= 0 2017-03-20
      end if

      if (Dj < 0 .or. Dj > 366) then
         Dj = 0
      end if

      if (D0 < 0 .or. D0 > 366) then
         D0 = 0
      end if

      if (ELV < 0 .or. ELV > 10000) then
         ELV = 0
      end if

      if (BUIEff < 0) then
        BUIEff=-1
      else
        BUIEff=0
      endif

      if (t < 0.) then
         t = -t
      end if

      if (t > 366.*24.) then
         t = 24.
      end if

      if (FFMC < 0. .or. FFMC > 101.) then
         FFMC = 0.
      end if

      if (DMC < 0. .or. DMC > 400.) then
        DMC = 0.
      end if

      if (DC < 0. .or. DC > 1200.) then
        DC = 0.
      end if

      if (Depth < 0. .or. Depth > 50.) then
        Depth = 0.
      end if

      if (BD < 0. .or. BD > 0.24) then
         BD = 0.
      end if

      if (ISI < 0. .or. ISI > 300.) then
         ISI = 0.
      end if

      if (BUI < 0. .or. BUI > 1000.) then
         BUI = 0.
      end if

      if (WS < 0. .or. WS > 300.) then
         WS = 0.
      end if

      if (WD < -2.*PI .or. WD > 2.*PI) then
         WD = 0.
      end if

      if (GS < 0. .or. GS > 200.) then
         GS = 0.
      end if

      if (Aspect < -2*PI .or. Aspect > 2*PI) then
         GS = 0.
      end if

      if (PC < 0. .or. PC > 100.) then
         PC = 50.
      end if

      if (PDF < 0. .or. PDF > 100.) then
         PDF = 35.
      end if

      if (GC <= 0. .or. GC > 100.) then
         GC = 95.
      end if

      if (GFL <= 0. .or. GFL > 10.) then
         GFL = 0.35  ! changed from 0.3 to 0.35, pg 6 - 2009
      end if

      if (LAT < -90. .or. LAT > 90.) then
         LAT = 0.
      end if

      if (LON < -180. .or. LON > 360.) then
         LON = 0.
      end if

      if (FMC <= 0. .or. FMC > 120.) then
         FMC = 0.
      end if

      if (theta < -2*PI .or. theta > 2*PI) then
         theta = 0.
      end if

      if (SD <0. .or. SD > 100000.) then
         SD=-999.
      end if

      if (SH <0. .or. SH > 100.) then
         SH=-999.
      end if

!  Convert time from hours to minutes
      t = t*60.

!  Corrections to reorient WAZ, SAZ
      WAZ = WD + PI
      if (WAZ > 2*PI) then
        WAZ = WAZ - 2*PI
      end if

!  nb: BMW's data set appears to have aspect not saz
      SAZ = Aspect + PI
      if (SAZ > 2*PI) then
         SAZ = SAZ - 2*PI
      end if

!  Make LON positive for the Western Hemisphere
      LON = -LON

!  Enter FBP Calculations
      iFuel = FBPFuel(FuelType)
      if (iFuel==-1 .or. FFMC == 0. .or. BUI == 0.) then
         fbp%ROS   = 0.
         fbp%FROS  = 0.
         fbp%BROS  = 0.
         fbp%TROS  = 0.
         fbp%HROSt = 0.
         fbp%FROSt = 0.
         fbp%BROSt = 0.
         fbp%TROSt = 0.
         fbp%CFB   = 0.
         fbp%FCFB  = 0.
         fbp%BCFB  = 0.
         fbp%TCFB  = 0.
         fbp%HFI   = 0.
         fbp%FFI   = 0.
         fbp%BFI   = 0.
         fbp%TFI   = 0.
         fbp%TFC   = 0.
         fbp%FTFC  = 0.
         fbp%BTFC  = 0.
         fbp%TTFC  = 0.
         fbp%SFC   = 0.
         fbp%TI    = -999.
         fbp%FTI   = -999.
         fbp%BTI   = -999.
         fbp%TTI   = -999.
         fbp%LB    = -999.
         fbp%RAZ    = -999.
         fbp%WSV    = -999.
         iStatus = -1    ! fail

         return
      end if

      CFB=0.

!  presently, we do not accept a zero BD, use near zero if necessary
      if (BD < 0.0 .or. BD > 0.5) then
         BD = 1000. * rhoBs(iFuel)
      else
         BD = 1000. * BD        ! NOTE: BD used in FFFC is kg/m^3 (not g/cm^3)
      end if

!  presently, we do not accept a zero Depths, use near zero if necessary
      if (Depth < 0.0 .or. Depth > 50.0) then
         Depth = depths(iFuel)
      end if

!  presently, we do not accept a zero CBH use near zero if necessary
      if (CBH <= 0. .or. CBH > 50.) then
         if (iFuel==C6 .and. SD > 0. .and. SH > 0.) then
            CBH = -11.2 + 1.06*SH + 0.00170*SD                          !  (91) - 2009
            if (CBH <0.) then
               CBH = 0.
            end if
         else
            CBH = CBHs(iFuel)
         end if
      end if

!  presently, we do not accept a zero CFL, use near zero if necessary
      if (CFL < 0.0 .or. CFL > 2.0) then
         CFL = CFLs(iFuel)
      end if

!  presently, we do not accept a zero SFL, use near zero if necessary
      if (SFL < 0.0 .or. SFL > 200.0) then
         SFL = SFLs(iFuel)
      end if

!  presently, we do not accept a zero FFL, use near zero if necessary
      if (FFL < 0.0 .or. FFL > 20.0) then
         FFL = FFLs(iFuel)
      end if

!  presently, we do not accept a zero WFL, use near zero if necessary
      if (WFL < 0.0 .or. WFL > 50.0) then
         WFL = WFLs(iFuel)
      end if

      SFCmax = SFLs(iFuel)+WFLs(iFuel)+FFLs(iFuel) ! set maximum SFC as the FBP defaults
      if (iFuel==O1a .or. iFuel==O1b) then
         SFCmax = 0.35
      end if

      if ((FMC <= 0 .or. FMC > 120.) .and. CFL > 0.) then
         call FMCcalc(LAT, LON, ELV, Dj, D0, FMC)
      else
         FMC = 0
      end if

      if (method <= 1 ) then                ! use default FBP SFLs

         SFL = SFLs(iFuel)
         WFL = WFLs(iFuel)
         FFL = FFLs(iFuel)
!         GFL = 0.35

         SFC = SFCcalc(iFuel, FFMC, BUI, PC, GFL, SFL, FFL, WFL)

      else if (method==2) then              ! use the spatial SFL as provided by Dan Thompson
         SFC = SFCcalc(iFuel, FFMC, BUI, PC, GFL, SFL, FFL, WFL)

      else if (method==3) then              !  use default SFLs but limit SFC to spatial SFL
         SFC = SFCcalc(iFuel, FFMC, BUI, PC, GFL, SFL, FFL, WFL)

         if (SFC > SFCmax) SFC = SFCmax

      else if (method==4) then       ! use de Groot et al (2007) with fuels

         SFC = SFCcalc(iFuel, FFMC, BUI, PC, GFL, SFL, FFL, WFL)

         if (iFuel==C1 .or. iFuel==C2 .or. iFuel==C3 .or. iFuel==C4 .or. iFuel==D1 ) then
            SFC = FFFCcalc(iFuel, Depth, BD, SFL, FFMC, DMC, DC, BUI)
         end if

         if (iFuel==M1 .or. iFuel==M2 ) then
            SFC = (PC/100. * FFFCcalc(C2, Depth, BD, SFL, FFMC, DMC, DC, BUI)) &
                + ((100. - PC)/100 * FFFCcalc(D1, Depth, BD, SFL, FFMC, DMC, DC, BUI) )
         end if


         if (SFC > SFCmax) SFC = SFCmax

      else ! if (method==5)                 ! use de Groot et al (2007) without fuels
         SFC = FFFCcalc(nofuel, Depth, BD, SFL, FFMC, DMC, DC, BUI)
      end if
!!! END

      if (GS > 0. .and. FFMC > 0.) then
         call Slopecalc(iFuel, FFMC, BUI, WS, WAZ, GS, &
                SAZ, FMC, SFC, PC, PDF, GC, CBH, &
                RAZ, WSV, BUIEff)
      else
         WSV = WS
         RAZ = WAZ
      end if

      if (FFMC > 0.) then
         ISI = ISICalc(FFMC, WSV)
      end if

      if (iFuel==C6) then
         call C6calc(iFuel, ISI, BUI, FMC, SFC, CBH,  ROS, CFB, RSC, BUIEff)!  We use C6calc to calculate CFB
      else
         ROS = ROScalc(iFuel, ISI, BUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)
         if (CFL > 0.) then
            CFB  = CFBcalc(iFuel, FMC, SFC, ROS, CBH)
         else
            CFB = 0.
         end if
      end if

      LB = LBcalc(iFuel, WSV)

      if (Accel.ne.0) then ! line source, no acceleration
         LBt = LB
      else
         LBt = LBtcalc(iFuel, LB, t, CFB)
      end if

      BROS = BROScalc(iFuel, FFMC, BUI, WSV, FMC, SFC, PC, PDF, GC, CBH, BUIEff)
      FROS = FROScalc(ROS, BROS, LB)

!  TROS is the rate of spread towards angle theta
      E = sqrt(1.-1./LB/LB)      !  eccentricity
      TROS= ROS * (1.-E)/(1.-E*cos(theta - RAZ)) !  note: this method uses the focus as the ignition point

!   TROS = ROSthetacalc(ROS, FROS, BROS, theta) ! I have personally rejected this approach in favour of simply using the focus

      if(Accel.ne.0) then ! line source, no acceleration
         ROSt  = ROS
         FROSt = FROS
         BROSt = BROS
         TROSt = TROS
      else
         ROSt  = ROStcalc(iFuel, ROS, t, CFB)
         BROSt = ROStcalc(iFuel, BROS, t, CFB)
         FROSt = FROScalc(ROSt, BROSt, LBt)
!        TROSt = ROSthetacalc(ROSt, FROSt, BROSt, theta)

         E = sqrt(1.-1./LBt/LBt)      !  eccentricity
         TROSt= ROSt * (1.-E)/(1.-E*cos(theta - RAZ)) !  note: this method uses the focus as the ignition point
      end if

      if (CFL == 0.) then
         CFB = 0.
         FCFB = 0.
         BCFB = 0.
         TCFB = 0.
      else
         if (iFuel==C6) then
            FCFB = 0.
            BCFB = 0.
            TCFB = 0.
         else
!  equilibrium values
            CFB  = CFBcalc(iFuel, FMC, SFC, ROS, CBH)
            FCFB = CFBcalc(iFuel, FMC, SFC, FROS, CBH)
            BCFB = CFBcalc(iFuel, FMC, SFC, BROS, CBH)
            TCFB = CFBcalc(iFuel, FMC, SFC, TROS, CBH)
         end if
      end if

      TFC  = TFCcalc(iFuel, CFL, CFB, SFC, PC, PDF)
      FTFC = TFCcalc(iFuel, CFL, FCFB, SFC, PC, PDF)
      BTFC = TFCcalc(iFuel, CFL, BCFB, SFC, PC, PDF)
      TTFC = TFCcalc(iFuel, CFL, TCFB, SFC, PC, PDF)

!  equilibrium values
      HFI = FIcalc(TFC,  ROS)
      FFI = FIcalc(FTFC, FROS)
      BFI = FIcalc(BTFC, BROS)
      TFI = FIcalc(TTFC, TROS)


      CONTINUE

!  For now...
      TI = 0.
      FTI = 0.
      BTI = 0.
      TTI = 0.

      fbp%ROS   = ROS
      fbp%FROS  = FROS
      fbp%BROS  = BROS
      fbp%TROS  = TROS

      if (fbp%t < 0) then
         fbp%HROSt = -ROSt
         fbp%FROSt = -FROSt
         fbp%BROSt = -BROSt
         fbp%TROSt = -TROSt
         fbp%CFB   = -CFB
      else
         fbp%HROSt = ROSt
         fbp%FROSt = FROSt
         fbp%BROSt = BROSt
         fbp%TROSt = TROSt
      end if

      fbp%CFB  = CFB
      fbp%FCFB = FCFB
      fbp%BCFB = BCFB
      fbp%TCFB = TCFB
      fbp%HFI  = HFI
      fbp%FFI  = FFI
      fbp%BFI  = BFI
      fbp%TFI  = TFI
      fbp%TFC  = TFC
      fbp%FTFC = FTFC
      fbp%BTFC = BTFC
      fbp%TTFC = TTFC
      fbp%SFC  = SFC
      fbp%TI   = TI
      fbp%FTI  = FTI
      fbp%BTI  = BTI
      fbp%TTI  = TTI
      fbp%LB   = LB
      fbp%RAZ  = RAZ * 180./PI
      fbp%WSV  = WSV
      fbp%FMC  = FMC
      fbp%D0   = D0

99    return
   end function FBPCalc_f90


! **********************************************

!   Determine and index for the FBP fuel type used throughout the program
!   Note that the order can be adjusted by varying the order in the static variables

! **********************************************

   integer (kind=4) function FBPFuel(FuelType) result(iFuel)
      implicit none
      character(len=FUEL_SIZE), intent(in) :: FuelType

      integer(kind=4) ::  i
      character(len=FUEL_SIZE) temp

      if (trim(FuelType) == "O1") then
         temp = "O1b"
      else
         temp = FuelType
      end if

      iFuel = -1    ! fail
      do i = 0, MAX_FUELS
         if (trim(FuelType) == Fuels(i) .and. &
             trim(FuelType) .ne. "WA" .and. &
             trim(FuelType) .ne. "NF") then
            iFuel = i
         end if
      end do

      return
   end function FBPFuel

! **********************************************

!   Foliar Moisture Content (FMC) calculation

!   Note that 0.5 is added before the integer conversion in equations 2 and 4
!   Note that equations 1 and 3 use positive longitude values for Canada

! **********************************************

   subroutine FMCcalc(LAT, LON, ELV, Dj, D0, FMC)
      implicit none
      real(kind=4),    intent(in) :: LAT, LON
      integer(kind=4), intent(in) :: ELV, Dj
      integer(kind=4), intent(inout) :: D0
      real(kind=4),    intent(inout) :: FMC

      real(kind=4) :: LATN
      integer(kind=4) :: ND

      FMC = -1.

      if (D0 <= 0) then     !  if D0, date of min FMC, is not known then D0 <= 0
         if (ELV <= 0) then
            LATN = 46.0 + 23.4 * exp(-0.0360 *(150. - LON))             ! (1)
            D0 = int(151. * LAT/LATN + 0.5)                             ! (2) +0.5
         else
            LATN = 43. + 33.7 * exp(-0.0351*(150 - LON))                ! (3)
            D0 = int(142.1 * LAT/LATN + (0.0172 * ELV) + 0.5)           ! (4) +0.5
         end if
      end if

      ND = abs(Dj - D0)       ! (5)

      if (ND < 30) then
         FMC = 85. + 0.0189 * ND * ND                                   ! (6)
      else
         if (ND < 50) then
            FMC = 32.9 + 3.17 * ND - 0.0288 * ND * ND                   ! (7)
         else
            FMC  = 120.                                                 ! (8)
         end if
      end if

      return
   end subroutine FMCcalc

! **********************************************
!   Surface Fuel Consumption (SFC) calculation
! **********************************************

   recursive function SFCcalc(iFuel, FFMC, BUI, PC, GFL, SFL, FFL, WFL) result(SFC)
      implicit none
      integer(kind=4), intent(in) :: iFuel
      real(kind=4),    intent(in) :: FFMC, BUI, PC, GFL, SFL, FFL, WFL

      real(kind=4) :: FFC, WFC, SFC

      SFC = -1.
      FFC = 0.
      WFC = 0.


! NOTE: Equations have been modified to allow for SFL, FFL and WFL variables
!       Default values are stored in SFLs(), FFLs(), WFLs()

      select case (iFuel)
         case (C1)
!           SFC = 1.5 * (1. - exp(-0.230 * (FFMC - 81.)))               ! (9)
            if (FFMC > 84.) then
               SFC = SFL/2. + SFL/2.*(1.-exp(-0.23*(FFMC-84.)))**0.5    ! (9a) - 2009
            else
               SFC = SFL/2. - SFL/2.*(1.-exp(-0.23*(84.-FFMC)))**0.5    ! (9b) - 2009 (note the reversal of FFMC to avoid negative sqrt)
            end if

         case (C2, M3, M4)
            SFC = SFL * (1. - exp(-0.0115 * BUI))                       ! (10)

         case(C3, C4)
            SFC = SFL * (1. - exp(-0.0164*BUI))**2.24                   ! (11)

         case(C5, C6)
            SFC = SFL * (1. - exp(-0.0149*BUI))**2.48                   ! (12)

         case(C7)
            if (FFMC> 70.) then
               FFC = FFL * (1. - exp(-0.104 * (FFMC - 70.)))            ! (13)
            else
               FFC = 0.
            end if
            WFC = WFL * (1. - exp(-0.0201 * BUI))                       ! (14)
            SFC = FFC + WFC                                             ! (15)

         case(D1)
            SFC = 1.5 * (1. - exp(-0.0183 * BUI))                       ! (16)

         case(M1, M2)
            SFC = (PC*0.01 * SFCcalc(C2, FFMC, BUI, PC, GFL, &
                  SFLs(C2), FFL, WFL)) &
            +((100. - PC)*0.01*SFCcalc(D1, FFMC, BUI, PC, GFL, &
                  SFLs(D1), FFL, WFL))                                  ! (17)

         case(O1a, O1b)
            SFC = GFL                                                   ! (18)

         case(S1)
            FFC = FFL * (1. - exp(-0.025*BUI))                          ! (19)
            WFC = WFL * (1. - exp(-0.034*BUI))                          ! (20)
            SFC = FFC + WFC                                             ! (25)

         case(S2)
            FFC = FFL * (1. - exp(-0.013*BUI))                          ! (19)
            WFC = WFL * (1. - exp(-0.060*BUI))                          ! (20)
            SFC = FFC + WFC                                             ! (25)

         case(S3)
            FFC = FFL * (1. - exp(-0.0166*BUI))                         ! (19)
            WFC = WFL * (1. - exp(-0.0210*BUI))                         ! (20)
            SFC = FFC + WFC                                             ! (25)

      end select

      return
   end function SFCcalc

! Forest Floor Fuel Consumption (de Groot et al 2009)
! this is a test of the FFFC as a substitute for SFC

   recursive function FFFCcalc(iFuel, Depth, BD, Load, FFMC, DMC, DC, BUI) result(FFFC)
      implicit none
      integer(kind=4), intent(in) :: iFuel
      real(kind=4),    intent(in) :: Depth, BD, Load, FFMC, DMC, DC, BUI

      real(kind=4) :: FFFC


! The general equations: n = 128                                                                ! r^2
      if (DC > 0. .and. Depth > 0. .and. Load > 0.) then
         FFFC = 1.1866 * exp(-7.388 + 0.754*log(DC) + 0.691*log(Depth) + 0.608*log(BD))         ! 0.796
      else if (DC > 0. .and. Load > 0.) then
         FFFC = 1.1852 * exp(-4.252 + 0.710*log(DC) + 0.671*log(Load))                          ! 0.795
      else if (Load > 0.) then
         FFFC = 1.2384 * exp(-0.882 + 1.082*log(Load))                                          ! 0.741
      else if (DC > 0.) then
         FFFC = 1.2821 * exp(-7.672 + 1.478*log(DC))                                            ! 0.699
      end if


! These are fuel specific equations.
!   Though some perform poorer than the general equations, they are used when applicable
      select case (iFuel)
         case (C1) ! n = 6
            if (FFMC > 0.) then
               FFFC = -6.142 + 0.083*FFMC                                                       ! 0.856
            end if

        case(C2) ! n = 30
            if (Load > 0. .and. DMC > 0.) then
               FFFC = 0.721 + 0.187*Load + 0.030*DMC                                            ! 0.206
            end if

        case(C3, C4) ! n = 64
            if (Depth > 0. .and. BD > 0. .and. DC > 0.) then
               FFFC = -0.965 + 0.181*Depth + 0.012*BD + 0.003*DC                                ! 0.429
            else if (DC > 0. .and. Load > 0.) then
               FFFC = 1.0959 * exp(-3.486 + 0.612*log(DC) + 0.484*log(Load))                    ! 0.639
            end if

        case(D1, D2) ! n = 18
            if (DC > 0. .and. BUI > 0.) then
                FFFC = 0.924 + 0.023*DC - 0.082*BUI                                             ! 0.906
            end if
      end select

! Just use the default equation
      if (iFuel < 0 .or. FFMC < 0) then
          if (DC > 0. .and. Load > 0.) then
                FFFC = 1.1852 * exp(-4.252 + 0.710*log(DC) + 0.671*log(Load))                   ! 0.795
          end if
      end if

      if (FFFC < 0.) FFFC = 0.


      return
   end function FFFCcalc

! **********************************************
!   Rate of Spread (ROS) calculation
! **********************************************

   recursive function ROScalc(iFuel, ISI, BUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)  result(ROS)
      implicit none
      integer(kind=4), intent(in) :: iFuel, BUIEff
      real(kind=4),    intent(in) :: ISI, BUI, FMC, SFC, PC, PDF, GC, CBH

      real(kind=4) :: RSI, RSI_M3, RSI_M4, CF, BE, ROS, CFB, RSC

      RSI = -1.

!  Note that only preliminary RSS calculations are D0ne for C6 in this routine
      select case (iFuel)
         case(C1, C2, C3, C4, C5, C7, D1, S1, S2, S3)
            RSI = a(iFuel) * (1. - exp(-b(iFuel) * ISI))**c(iFuel)      ! (26)

         case(M1)
            RSI = PC*0.01 * ROScalc(C2, ISI, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff) &
            + (100.-PC)*0.01 * ROScalc(D1, ISI, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)  ! (27)

         case(M2)
            RSI = PC*0.01 * ROScalc(C2, ISI, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff) &
            +0.2*(100.-PC)*0.01 * ROScalc(D1, ISI, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)  ! (28)

         case(M3)
            RSI_M3 = a(iFuel) * (1. - exp(-b(iFuel) * ISI))**c(iFuel)   ! (30) - 2009

            RSI = PDF*0.01* RSI_M3 + &
            (1.-PDF*0.01)* ROScalc(D1, ISI, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)    ! (29) - 2009

         case(M4)
            RSI_M4 = a(iFuel) * (1. - exp(-b(iFuel) * ISI))**c(iFuel)   ! (32) - 2009

            RSI = PDF*0.01* RSI_M4 + &
            0.2*(1.-PDF*0.01)* ROScalc(D1, ISI, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)     ! (31) - 2009

         case(O1a, O1b)
            if (GC < 58.8) then
               CF = 0.005*(exp(0.061*GC)-1.)                            ! (35a) - 2009
            else
               CF = 0.176 + 0.02*(GC-58.8)                              ! (35b) - 2009
            end if
!  RSI has been substituted for ROS in eqn 36
            RSI = a(iFuel) * (1. - exp(-b(iFuel) * ISI))**c(iFuel) * CF ! (36)

      end select


      if (iFuel==C6) then
         call C6calc(iFuel, ISI, BUI, FMC, SFC, CBH,  ROS, CFB, RSC, BUIEff)    !  included here for completeness
      else
         BE = BEcalc(iFuel, BUI, BUIEff)
         ROS = BE*RSI                                                   ! (55)
      end if

      if (ROS <= 0.) then
         ROS = 0.000001
      end if

   end function ROScalc

! **********************************************
!   Initial Spread Index (ISI) calculation
! **********************************************

   real(kind=4) function ISICalc(FFMC, WSV) result(ISI)
      implicit none
      real(kind=4),    intent(in) :: FFMC, WSV

      real(kind=4) :: fW,m,fF

      m = 147.2*(101.-FFMC)/(59.5+FFMC)                                 ! (46)
      fF = 91.9*exp(-0.1386*m)*(1.+ m**5.31/4.93e7)                     ! (45)

      if (WSV < 40.) then
         fW = exp(0.05039*WSV)                                          ! (53)
      else
         fW = 12. * (1. - exp(-0.0818 * (WSV-28.)))                     ! (53a)
      end if

      ISI = 0.208*fW*fF                                                 ! (52)

      return
   end function ISICalc

! **********************************************
!   Effect of Slope on Rate of Spread (ROS)
! **********************************************

   subroutine Slopecalc(iFuel, FFMC, BUI, WS, WAZ, GS, &
               SAZ, FMC, SFC, PC, PDF, GC, CBH, &
               RAZ, WSV, BUIEff)
      implicit none
      integer(kind=4), intent(in)  :: ifuel, BUIEff
      real(kind=4),    intent(in)  :: FFMC, BUI, WS, WAZ, GS, SAZ, &
                                      FMC, SFC, PC, PDF, GC, CBH
      real(kind=4),    intent(out) :: RAZ, WSV

      real(kind=4) :: SF, RSZ, RSF, RSF_C2, RSF_D1, RSF_M3, RSF_M4, &
                      PDF100, ISZ, ISF, ISF_C2, ISF_D1, ISF_M3, ISF_M4, &
                      CF, m, fF, WSE, WSX, WSY

      if (GS >= 70.) then
         SF=10.
      else
         SF = exp(3.533 * (GS*0.01)**1.2)                               ! (39)
      end if

      ISZ = ISICalc(FFMC, 0.)
      RSZ = ROScalc(iFuel, ISZ, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)
      RSF = RSZ * SF                                                    ! (40)

      select case (iFuel)
         case(C1, C2, C3, C4, C5, C6, C7, D1, S1, S2, S3)
            if ((1. - (RSF/a(iFuel))**(1./c(iFuel))) >= 0.01) then
               ISF = log(1. - (RSF/a(iFuel))**(1./c(iFuel)))/(-b(iFuel))! (41a) - 2009
            else
               ISF = log(.01)/(-b(iFuel))                               ! (41b) - 2009
            end if

         case (M1, M2)
            RSZ = ROScalc(C2, ISZ, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)
            RSF_C2 = RSZ * SF                                              ! (40)
            RSZ = ROScalc(D1, ISZ, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)
            RSF_D1 = RSZ * SF                                              ! (40)

            if ((1. - (RSF_C2/a(C2))**(1./c(C2))) >= 0.01) then
               ISF_C2 = log(1. - (RSF_C2/a(C2))**(1./c(C2)))/(-b(C2))      ! (41a) - 2009
            else
               ISF_C2 = log(.01)/(-b(C2))                                  ! (41b) - 2009
            end if

            if ((1. - (RSF_D1/a(D1))**(1./c(D1))) >= 0.01) then
               ISF_D1 = log(1. - (RSF_D1/a(D1))**(1./c(D1)))/(-b(D1))      ! (41a) - 2009
            else
               ISF_D1 = log(.01)/(-b(D1))                                  ! (41b) - 2009
            end if

            ISF = PC*0.01*ISF_C2+(1.-PC*0.01)*ISF_D1                          ! (42a) - 2009

         case (M3)
            PDF100=100.
            RSZ = ROScalc(M3, ISZ, NoBUI, FMC, SFC, PC, PDF100, GC, CBH, BUIEff)
            RSF_M3 = RSZ * SF                                              ! (40)
            RSZ = ROScalc(D1, ISZ, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)
            RSF_D1 = RSZ * SF                                              ! (40)

            if ((1. - (RSF_M3/a(M3))**(1./c(M3))) >= 0.01) then
               ISF_M3 = log(1. - (RSF_M3/a(M3))**(1./c(M3)))/(-b(M3))      ! (41a) - 2009
            else
               ISF_M3 = log(.01)/(-b(M3))                                  ! (41b) - 2009
            end if

            if ((1. - (RSF_D1/a(D1))**(1./c(D1))) >= 0.01) then
               ISF_D1 = log(1. - (RSF_D1/a(D1))**(1./c(D1)))/(-b(D1))      ! (41a) - 2009
            else
               ISF_D1 = log(.01)/(-b(D1))                                  ! (41b) - 2009
            end if

            ISF = PDF*0.01*ISF_M3 + (1.-PDF*0.01)*ISF_D1                   ! (42b) - 2009

         case (M4)
            PDF100=100.
            RSZ = ROScalc(M4, ISZ, NoBUI, FMC, SFC, PC, PDF100, GC, CBH, BUIEff)
            RSF_M4 = RSZ * SF                                              ! (40)
            RSZ = ROScalc(D1, ISZ, NoBUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)
            RSF_D1 = RSZ * SF                                              ! (40)

            if ((1. - (RSF_M4/a(M4))**(1./c(M4))) >= 0.01) then
               ISF_M4 = log(1. - (RSF_M4/a(M4))**(1./c(M4)))/(-b(M4))      ! (41a) - 2009
            else
               ISF_M4 = log(.01)/(-b(M4))                                  ! (41b) - 2009
            end if

            if ((1. - (RSF_D1/a(D1))**(1./c(D1))) >= 0.01) then
               ISF_D1 = log(1. - (RSF_D1/a(D1))**(1./c(D1)))/(-b(D1))      ! (41a) - 2009
            else
               ISF_D1 = log(.01)/(-b(D1))                                  ! (41b) - 2009
            end if

            ISF = PDF*0.01*ISF_M4 + (1.-PDF*0.01)*ISF_D1                   ! (42c) - 2009

         case (O1a, O1b)
            if (GC < 58.8) then
               CF = 0.005*(exp(0.061*GC)-1.)                               ! (35a) - 2009
            else
               CF = 0.176 + 0.02*(GC-58.8)                                 ! (35b) - 2009
            end if

            if ((1. - (RSF/(CF*a(iFuel)))**(1./c(iFuel))) >= 0.01) then
               ISF = log(1. - (RSF/(CF*a(iFuel)))**(1./c(iFuel)))/(-b(iFuel)) ! (43a) - 2009
            else
               ISF = log(0.01)/(-b(iFuel))                                 ! (43b) - 2009
            end if

      end select

      m = 147.2*(101.-FFMC)/(59.5+FFMC)                                 ! (46)
      fF = 91.9*exp(-0.1386*m)*(1.+ m**5.31/4.93e7)                     ! (45)

!   WSE = log(ISF/(0.208 * fF))/0.05039                                 ! (44)

      WSE = 1./0.05039*log(ISF/(0.208 * fF))                            ! (44a) , 44d- 2009
      if (WSE > 40) then                                                ! (44e) - 2009
         if (ISF < (0.999*2.496*fF) ) then
            WSE = 28.-(1./0.0818*log(1.-ISF/(2.496*fF)))                ! (44b) - 2009
         else
            WSE = 112.45                                                ! (44c) - 2009
         end if
      end if

      WSX = WS*sin(WAZ) + WSE*sin(SAZ)                                  ! (47)
      WSY = WS*cos(WAZ) + WSE*cos(SAZ)                                  ! (48)

      WSV = sqrt(WSX*WSX + WSY*WSY)                                     ! (49)
      RAZ = acos(WSY/WSV)   !  in radians                               ! (50)
      if (WSX < 0.) then
         RAZ = 2*PI - RAZ                                               ! (51)
      end if

      return
   end subroutine Slopecalc

! **********************************************
!   Buildup Index (BUI) Effect
! **********************************************

   real(kind=4) function BEcalc(iFuel, BUI, BUIEff) result(BE)
      implicit none
      integer(kind=4), intent(in) :: iFuel, BUIEff
      real(kind=4),    intent(in) :: BUI

      if (BUIEff==0 .and. BUI > 0. .and. BUIo(iFuel) > 0.) then
         BE = exp(50.*log(q(iFuel))*(1./BUI - 1./BUIo(iFuel)))          ! (54)
      else
         BE = 1.
      end if

      return
   end function BEcalc

! **********************************************
!   Crown Fraction Burned (CFB) calculation
! **********************************************

   real(kind=4) function CFBcalc(iFuel, FMC, SFCin, ROS, CBH) result(CFB)
      implicit none
      integer(kind=4), intent(in) :: iFuel
      real(kind=4),    intent(in) :: FMC, SFCin, ROS, CBH

      real(kind=4) :: SFC, CSI, RSO

      SFC = SFCin

!  I have no idea why BMW has this in there -- must comment out to agree with BMW 2009
!      if (iFuel==C6) then
!         SFC = 2.0
!      end if

      CFB = 0.

      if (SFC > 0.) then
        CSI = 0.001 * CBH**1.5 * (460. + 25.9*FMC)**1.5                 ! (56)
        RSO = CSI/(300.*SFC)                                            ! (57)
        if (ROS > RSO) then
            CFB = 1. - exp(-0.23*(ROS - RSO))                           ! (58)
        end if
      end if
      return
   end function CFBcalc

! **********************************************
!   C6 specific calculation
! **********************************************

   subroutine C6calc(iFuel, ISI, BUI, FMC, SFC, CBH, ROS, CFB, RSC, BUIEff)
      implicit none
      integer(kind=4), intent(in)  :: iFuel, BUIEff
      real(kind=4),    intent(in)  :: ISI, BUI, FMC, SFC, CBH
      real(kind=4),    intent(out) :: ROS, CFB, RSC

      real(kind=4) :: T, h, FME, RSI, RSS, FMEavg

      FMEavg = 0.778                                                    !  page 37
      T = 1500. - 2.75 * FMC                                            ! (59)
      h = 460. + 25.9 * FMC                                             ! (60)
      FME = (1.5 - 0.00275 * FMC)**4./(460. + 25.9*FMC) * 1000.         ! (61)
      RSI = 30. * (1. - exp(-0.08 * ISI))**3.0                          ! (62)
      RSS = RSI * BEcalc(iFuel, BUI, BUIEff)                            ! (63)
      RSC = 60. * (1. - exp(-0.0497*ISI)) * FME/FMEavg                  ! (64)

      if (RSC > RSS) then
         CFB = CFBcalc(iFuel, FMC, SFC, RSS, CBH)
         ROS = RSS + CFB*(RSC-RSS)                                      ! (65)
      else
         CFB=0.
         ROS = RSS
      end if

      return
   end subroutine C6calc

! **********************************************
!   Total Fuel Consumption (TFC) calculation
! **********************************************

   real(kind=4) function TFCcalc(iFuel, CFL, CFB, SFC, PC, PDF) result(TFC)
      implicit none
      integer(kind=4), intent(in) :: iFuel
      real(kind=4),    intent(in) :: CFL, CFB, SFC, PC, PDF
      real(kind=4) :: CFC

      CFC = CFL * CFB                                                   ! (66), (66a) - 2009

      select case (iFuel)
         case(M1, M2)
            CFC = PC*0.01*CFC                                           ! (66b) - 2009

         case(M3, M4)
            CFC = PDF*0.01*CFC                                          ! (66c) - 2009

      end select

      TFC = SFC + CFC                                                   ! (67)

      return
   end function TFCcalc

! **********************************************
!   Fire Intensity (FI) calculation
! **********************************************

   real(kind=4) function FIcalc(FC, ROS) result(FI)
      implicit none
      real(kind=4),    intent(in) :: FC, ROS

      FI = 300. * FC * ROS                                              ! (69)

      return
   end function FIcalc


! **********************************************
!   Back Rate of Spread (BROS) calculation
! **********************************************

   real(kind=4) function BROScalc (iFuel, FFMC, BUI, WSV, FMC, SFC, PC, PDF, GC, CBH, BUIEff) result(BROS)
      implicit none
      integer(kind=4), intent(in) :: iFuel, BUIEff
      real(kind=4),    intent(in) :: FFMC, BUI, WSV, FMC, SFC, PC, PDF, GC, CBH

      real(kind=4) :: m, fF, BfW, BISI

      m = 147.2*(101.-FFMC)/(59.5+FFMC)                                 ! (46)
      fF = 91.9*exp(-0.1386*m)*(1.+ m**5.31/4.93e7)                     ! (45)
      BfW = exp(-0.05039*WSV)                                           ! (75)
      BISI = 0.208*BfW*fF                                               ! (76)

!  Note the BUI effect is captured in ROScalc
      BROS =  ROScalc(iFuel, BISI, BUI, FMC, SFC, PC, PDF, GC, CBH, BUIEff)     ! (77)

      return
   end function BROScalc

! **********************************************
!   Rate of Spread over time (ROSt) calcuations
! **********************************************

   real(kind=4) function ROStcalc(iFuel, ROSeq, t, CFB) result(ROSt)
      implicit none
      integer(kind=4), intent(in) :: iFuel
      real(kind=4),    intent(in) :: ROSeq, t, CFB

      real(kind=4) :: alpha

      select case (iFuel)
         case(C1, O1a, O1b, S1, S2, S3)
            alpha = 0.115                                               !  page 41
         case default
            alpha = 0.115 - 18.8 * CFB**2.5 * exp(-8.* CFB)             ! (72)
      end select

      ROSt = ROSeq * (1. - exp (-alpha * t))                            ! (70)

      return
   end function ROStcalc

! **********************************************
!   Length to Breadth (LB) calculation
! **********************************************

   real(kind=4) function LBcalc(iFuel, WSV) result(LB)
      implicit none
      integer(kind=4), intent(in) :: iFuel
      real(kind=4),    intent(in) :: WSV

      select case (iFuel)
         case(O1a, O1b)
            if (WSV >= 1.0) then
               LB = 1.1 * WSV**0.464                                    !  corrected from "+" to "*" in the errata 80
            else
               LB = 1.0                                                 ! (81)
            end if
         case default
            LB = 1.0 + 8.729 * (1.-exp(-0.030*WSV))**2.155              ! (79)
      end select

      return
   end function LBcalc

! *************************************************
!   Length to Breadth over time (LBt) calculation
!   this is a new subroutine to account for the discussion in 3.6.1
! *************************************************

   real(kind=4) function LBtcalc(iFuel, LB, t, CFB) result(LBt)
      implicit none
      integer(kind=4), intent(in) :: iFuel
      real(kind=4),    intent(in) :: LB, t, CFB

      real(kind=4) :: alpha

      select case (iFuel)
         case(C1, O1a, O1b, S1, S2, S3)
            alpha = 0.115                                               !  page 41
         case default
            alpha = 0.115 - 18.8 * CFB**2.5 * exp(-8.* CFB)             ! (72)
      end select

      LBt = (LB -1.) * (1. - exp(-alpha*t)) + 1.                        ! (81) - 2009

      return
   end function LBtcalc

! **********************************************
!   Flank Rate of Spread (FROS) calculation
! **********************************************

   real(kind=4) function FROScalc(ROS, BROS, LB) result(FROS)
      implicit none
      real(kind=4),    intent(in) :: ROS, BROS, LB

      FROS = (ROS + BROS)/LB/2.                                         ! (89)

      return
   end function FROScalc

! ********************************************************
!   Rate of Spread at angle theta (ROStheta) calculation
! ********************************************************

   real(kind=4) function ROSthetacalc(ROS, FROS, BROS, theta) result(ROStheta)
      implicit none
      real(kind=4),    intent(in) :: ROS, FROS, BROS, theta

      real(kind=4) :: c, s

      c = cos(theta)
      s = sin(theta)

      if (c==0.) then
         c = cos(theta+.001)
      else
         ROStheta = (ROS - BROS)/(2.*c) + (ROS + BROS)/(2.*c)* &
            (FROS*c*sqrt(FROS*FROS * c*c + (ROS*BROS)*s*s) - &
            (ROS*ROS - BROS*BROS)/4.*s*s)/ &
            (FROS*FROS*c*c + ((ROS+BROS)/2.)*((ROS+BROS)/2.)*s*s)      ! (94) - 2009
      end if

      return
   end function ROSthetacalc


! **********************************************
!   FBP calculations test
! **********************************************

    subroutine FBPtest(fbp)
      implicit none
      type(fbp_type),  intent(inout) :: fbp

      integer(kind=4) :: i, iStatus, icase
      real (kind=4) :: SAZ, WAZ
      logical :: file_exists

      INQUIRE(FILE="fbp2009.inp", EXIST=file_exists)

      fbp%DMC    = -1                  ! added 2019-11-01
      fbp%DC     = -1                  ! added 2019-11-01
      fbp%Depth  = -1                  ! added 2019-11-01
      fbp%BD     = -1                  ! added 2019-11-01
      fbp%SFL    = -1                  ! added 2019-07-25
      fbp%FFL    = -1                  ! added 2019-07-25
      fbp%WFL    = -1                  ! added 2019-07-25
      fbp%FC_method = 1                ! added 2020-02-21



      IF (file_exists) THEN
         WRITE(6,*) "Table 5 (key elements)" ! lists out key elements from Table 5 of BMW - 2009

         write(6,*) "Case  WSV   RAZ    FMC   Do   SFC   CFC    TFC   CFB     ROS      HFI"
         OPEN (UNIT=1, FILE="fbp2009.inp", STATUS='old')


         DO 10  i=1, 20
            READ(1, *) icase,  fbp%FuelType, fbp%FFMC, fbp%BUI, fbp%WS, &
                fbp%WD, WAZ, fbp%GS, SAZ, fbp%LAT, fbp%LON, fbp%ELV, &
                fbp%Dj, fbp%D0, fbp%t, fbp%PC, fbp%PDF, fbp%GFL, fbp%C

            fbp%Accel = 0
            fbp%LON = -fbp%LON
            fbp%Aspect = SAZ-180.
            fbp%BUIEff = 0
            fbp%CBH = -1.
            fbp%CFL = -1.
            fbp%SFL = -1.
            fbp%FFL = -1.
            fbp%WFL = -1.
            fbp%t=(fbp%t)/60.
            fbp%FMC=-1

            fbp%DMC    = 50                  ! added 2019-11-01
            fbp%DC     = 200                  ! added 2019-11-01
            fbp%BUI = 60
            fbp%Depth  = 7                  ! added 2019-11-01
            fbp%BD     = 0.035                  ! added 2019-11-01
            fbp%SFL    = 10                  ! added 2019-07-25
            fbp%FC_method = 5                ! added 2020-02-21

            istatus = FBPCalc_f90(fbp)

            write(6,100) icase, fbp%WSV, fbp%RAZ, fbp%FMC, fbp%D0, &
            fbp%SFC, fbp%TFC-fbp%SFC, fbp%TFC, fbp%CFB, fbp%ROS, fbp%HFI
100         format(I3, f8.3, f6.1, f7.2, i4, &
                   f7.3, f6.3, f7.3, f6.3, f8.3, f10.2)
10       CONTINUE

         CLOSE(1)

! Repeat for Table 6
! Errata (acknowledged by BMW)
!   Case 6: FROS = 7.309, FFI = 2670.98
!   Case 20 FROS = 3.253, FFI = 2590.31

         WRITE(6,*)
         WRITE(6,*) "Table 6 (key elements)" ! lists out key elements from Table 6 of BMW - 2009

         write(6,*) "Case LB   LBt    HROSt   FROS     FFI     FROSt   BROS     BFI     BROSt"
         OPEN (UNIT=1, FILE="fbp2009.inp", STATUS='old')


         DO 20  i=1, 20
            READ(1, *) icase,  fbp%FuelType, fbp%FFMC, fbp%BUI, fbp%WS, &
                fbp%WD, WAZ, fbp%GS, SAZ, fbp%LAT, fbp%LON, fbp%ELV, &
                fbp%Dj, fbp%D0, fbp%t, fbp%PC, fbp%PDF, fbp%GFL, fbp%C

            fbp%Accel = 0
            fbp%LON = -fbp%LON
            fbp%Aspect = SAZ-180.
            fbp%BUIEff = 0
            fbp%CBH = -1.
            fbp%CFL = -1.
            fbp%t=(fbp%t)/60.
            fbp%FMC=-1

            istatus = FBPCalc_f90(fbp)

            write(6,200) icase, fbp%LB, &
               (fbp%HROSt+fbp%BROSt)/fbp%FROSt/2., &
               fbp%HROSt, &
               fbp%FROS, fbp%FFI, fbp%FROSt, &
               fbp%BROS, fbp%BFI, fbp%BROSt
200         format (I3, 2f6.3, f8.3, 2(f8.3, f9.2, f8.3))
20       CONTINUE

         CLOSE(1)

      ELSE
         fbp%FuelType="C2"
         fbp%Accel = 1
         fbp%Dj = 0
         fbp%D0 = 0
         fbp%ELV = -9
         fbp%BUIEff = 0
         fbp%t = 0
         fbp%FFMC = 90
         fbp%DMC = 30
         fbp%DC = 200
         fbp%ISI = 0
         fbp%BUI = 60
         fbp%WS = 10
         fbp%WD = 0
         fbp%GS = 0
         fbp%Aspect = 0
         fbp%PC = 50
         fbp%PDF = 35
         fbp%C = 80.
         fbp%GFL = 3.
         fbp%CBH = 7.
         fbp%CFL = -1.
         fbp%SFL = -1.
         fbp%FFL = -1.
         fbp%WFL = -1.
         fbp%LAT = 60.
         fbp%LON = -120.
         fbp%FMC = 0.
         fbp%theta = 0.
         fbp%SH = -999.
         fbp%SD = -999.

         istatus = FBPCalc_f90(fbp)

         WRITE(6,*) istatus, fbp%ROS, fbp%CFB, fbp%HFI

      END IF

30    return
   end subroutine FBPtest

   end module FBP_module

