!============================================================================!
!
! Projet / Project : GEM-MACH
! Fichier / File   : fwi95.f95
! Creation         : Kerry Anderson - September 17, 2019
! Description      : Canadian FOrest Fire Weather Index System
!
! This subroutine represents equations from:
!
! Van Wagner, C.E. and Pickett, T.L., 1985. Equations and FORTRAN program for 
!   the Canadian forest fire weather index system. For. Tehc Rep. 33.  
!   Canadian Forest Service, Ottawa, ON. 18 pp.
!
! Variable names and equation number within subroutines 
!   consistent with the report.
! 
! Variant of Le allowing day length based on latitude
!   included in DMC calculations through variable RefLat:
!        46 -- 46N: Canadian standard, latitude >= 30N
!        20 -- 20N: For 30 > latitude >= 10
!         0 -- EQ:  For 10 > latitude > -10
!       -20 -- 20S: For -10 > latitude >= -30
!       -40 -- 40S: For -30 > latitude
!        91 -- Use several categories listed above if
!              region crosses these bounds, based on latitude
!        92 -- Spatial calculation based on latitude 
!
! Variant of Lf allowing for seasonality of
!   hemisphere and equatorial zone (20oN to -20oN)
!   included in DC calculations through Lat variable
!
! **********************************************

module fwiindices !!!***** BEGINNING OF MODULE *****!!!
implicit none

real(4), save :: El(12),Le46N(12),Le20N(12),LeEQ(12),Le20S(12),Le40S(12)
real(4), save :: Fl(12),Lfn(12),Lfs(12),Lfe(12)
integer, save :: days(12),JD,RefLat

data El/6.5,7.5,9.0,12.8,13.9,13.9,12.4,10.9,9.4,8.0,7.0,6.0/
data Le46N/6.5,  7.5, 9.0, 12.8, 13.9, 13.9, 12.4, 10.9, 9.4,  8.0,  7.0,  6.0/
data Le20N/7.9,  8.4, 8.9,  9.5,  9.9, 10.2, 10.1,  9.7, 9.1,  8.6,  8.1,  7.8/
data LeEQ/9.0,  9.0, 9.0,  9.0,  9.0,  9.0,  9.0,  9.0, 9.0,  9.0,  9.0,  9.0/
data Le20S/10.1,  9.6, 9.1,  8.5,  8.1,  7.8,  7.9,  8.3, 8.9,  9.4,  9.9, 10.2/
data Le40S/11.5, 10.5, 9.2,  7.9,  6.8,  6.2,  6.5,  7.4, 8.7, 10.0, 11.2, 11.8/

data Fl/-1.6,-1.6,-1.6,0.9,3.8,5.8,6.4,5.0,2.4,0.4,-1.6,-1.6/
data Lfn/-1.6,-1.6,-1.6, 0.9, 3.8, 5.8, 6.4, 5.0, 2.4, 0.4,-1.6,-1.6/
data Lfs/ 6.4, 5.0, 2.4, 0.4,-1.6,-1.6,-1.6,-1.6,-1.6, 0.9, 3.8, 5.8/
data Lfe/ 1.4, 1.4, 1.4, 1.4, 1.4, 1.4, 1.4, 1.4, 1.4, 1.4, 1.4, 1.4/

data days/0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334/

Contains


!*** Subroutine FFMCcalc ***!
subroutine FFMCcalc(T,H,W,ro,Fo, F)
implicit none
    real(4), intent(in)  :: T,H,W,ro,Fo 
    real(4), intent(out) :: F

    real(4) mo,rf,mr,Ed,Ew,ko,kd,kl,kw,m    

    if (T > -99. .AND. H>=0. .AND. W>=0. .AND. ro >= 0. .AND. Fo >= 0.) then

        mo = (147.2*(101. - Fo))/(59.5 + Fo)                                    !*Eq.1*!
        if (ro > 0.5) then                          
            rf = ro - 0.5                                                       !*Eq.2*!
            if(mo <= 150.) then
                mr=mo+42.5*rf*exp(-100./(251.-mo))*(1. - exp(-6.93/rf))         !*Eq.3a*!
            else
                mr=(mo+42.5*rf*exp(-100./(251.-mo))*(1. - exp(-6.93/rf)))  &
                + (.0015*(mo - 150.)**2)*sqrt(rf)                            !*Eq.3b*!
            endif    
            if (mr > 250.) mr=250.
            mo = mr    
        endif  

        Ed=0.942*(H**0.679) + (11.*exp((H-100.)/10.))+0.18*(21.1-T)  &
                    *(1. - exp(-0.1150 * H))                                    !*Eq.4*!

        if(mo > Ed) then
            ko = 0.424*(1.-(H/100.)**1.7)+(0.0694*sqrt(W))*(1.- (H/100.)**8)    !*Eq.6a*!
            kd = ko * (0.581*exp(0.0365*T))                                     !*Eq.6b*!
            m  = Ed + (mo-Ed)/10.**kd                                           !*Eq.8*!
        else
            Ew = 0.618*(H**0.753) + (10.*exp((H-100.)/10.)) &
                + 0.18*(21.1-T)*(1. - exp(-0.115 * H))                         !*Eq.5*!
            if(mo < Ew) then  
                kl = 0.424*(1.-((100. - H)/100.)**1.7) &
                   + (0.0694*sqrt(W)) * (1. - ((100. - H)/100.)**8)        !*Eq.7a*!
                kw = kl * (0.581 * exp(0.0365 * T))                             !*Eq.7b*!
                m  = Ew - (Ew - mo)/10.**kw                                     !*Eq.9*!
            else
                m = mo
            endif
        endif

        F = (59.5 * (250. - m)) / (147.2 + m)                                   !*Eq.10*! 
        if (F  > 101.)  F = 101.
        if (F  <=  0.)  F =   0.             
        
    else
        F = -999.
    endif

    return
end subroutine 


!***** Subroutine DMCcalc  *****!
subroutine DMCcalc(T, H, ro, Po, I, JD, Lat, RefLat, P)
implicit none
    integer, intent(in)  :: I, JD, RefLat   
    real(4), intent(in)  :: T,H,ro,Po, Lat
    real(4), intent(out) :: P
   
    real(4) re,Mo,Mr,K,b,Pr
    real(4) latd, latr, alpha, omega, beta, gamma, dla, dlb, dl, Le
    
    real(4) E_ecc, E_obl, E_sidyr, PIOVER180
    
! Eccentricity from NASA-JPL Keplerian elements for A.D. 1800-2050:
!      http://ssd.jpl.nasa.gov/txt/p_elem_t1.txt (Earth-Moon barycenter)
    E_ecc = 0.01671123

! The following constants are taken from l'Observatoire de Paris, Earth
!   Orientation Centre (http://hpiers.obspm.fr/eop-pc/models/constants.html):
    E_obl   = 23.4334
    E_sidyr = 365.2564

    PIOVER180 = 3.1415926/180.

    if (T > -99. .AND. H>=0. .AND. ro >= 0. .AND. Po >= 0.) then
        latd = Lat;
        if (Lat > 60.0) latd = 60.0
        if (Lat < -60.0) latd = -60.0

        if (RefLat == 46 .OR. (RefLat == 91 .AND. latd >= 30)) then
            Le = Le46N(I)
        else if  (RefLat == 20 .OR. (RefLat == 91 .AND. latd < 30 .AND. latd >= 10)) then
            Le = Le20N(I)
        else if  (RefLat == 0  .OR. (RefLat == 91 .AND. latd < 10 .AND. latd >= -10)) then
            Le = LeEQ(I)
        else if  (RefLat == -20 .OR. (RefLat == 91 .AND. latd < -10 .AND. latd >= -30)) then
            Le = Le20S(I)
        else if  (RefLat == 20 .OR. (RefLat == 91 .AND. latd < -30)) then
            Le = Le40S(I)
        else if (RefLat == 92) then
            latr = latd * PIOVER180;             ! Latitude in radians
            alpha = E_obl * PIOVER180;           ! Maximum solar declination
            omega = 360.0 * PIOVER180/E_sidyr;   ! Angular velocity of earth around sun per day
            beta = omega * (JD-80.5) + 2. * E_ecc * (sin(JD*omega) - sin(80.5 * omega));
            gamma = sin(alpha) * sin(beta);
!        gamma = atan2(gamma/pow(1-pow(gamma, 2), 0.5), 1.0);
            gamma = atan2(gamma/((1-(gamma**2))**0.5), 1.0);
            dla = -1.0 * tan(latr) * tan(gamma);
!        dlb = atan2(pow(1-pow(dla, 2), 0.5), dla);
            dlb = atan2((1-(dla**2))**0.5, dla);
            dl = dlb * 2./(PIOVER180*15.);
            Le = dl - 3.; 

        end if
    
   b=0
   Pr=0
   re=0
   P=0
    

        if(ro <= 1.5) then 
            Pr = Po 
        else
            re = 0.92*ro - 1.27                         !*Eq.11*!
            Mo = 20. + exp(5.6348 - Po/43.43)           !*Eq.12*!

            if(Po <= 33.) then 
                b = 100. / (0.5 + 0.3*Po)               !*Eq.13a*!
            else if (Po <= 65.) then 
                b = 14. - 1.3*log(Po)                   !*Eq.13b*!   
            else
                b =  6.2 * log(Po) - 17.2               !*Eq.13c*!
            endif

            Mr = Mo + (1000.*re) / (48.77 + b*re)       !*Eq.14*! 
            Pr = 244.72 - 43.43 * log(Mr-20.)           !*Eq.15*!
        endif

        if (T >= -1.1) then
            K = 1.894*(T+1.1) * (100.-H) * (Le*1.E-6)   !*Eq.16*!
        else
            K = 0.                                      !*Eq.17*!
        endif

        if (Pr <  0.) Pr = 0.

        P = Pr + 100.*K                                 !*Eq.17*!
        if(P<=0.) P=0.
        
    else
        P = -999.
    endif
    
    return
end subroutine  


!*****   Subroutine DCcalc        *****!
subroutine DCcalc (T, ro, D0, I, Lat, D)
implicit none
    real(4), intent(in)    :: T, ro
    real(4), intent(inout) :: D0, Lat
    real(4), intent(out)   :: D
    integer,intent(in)  :: I 
    
    real(4) rd,Qo,Qr,V,Dr,Lf

    if (T > -99. .AND. ro>=0. .AND. D0>=0.) then

        if (Lat > 20.) then
            Lf = Lfn(I)
        else if (Lat < -20) then
            Lf = Lfs(I)
        else
            Lf = Lfe(I)
        endif    

        if (ro > 2.8) then  
            rd = 0.83*ro - 1.27                     !*Eq.18*!
            Qo = 800. * exp(-D0/400.)               !*Eq.19*!
            Qr = Qo + 3.937*rd                      !*Eq.20*!
            Dr = 400.*log(800./Qr)                  !*Eq.21*!
            
            if(Dr > 0.) then
                D0 = Dr
            else
                D0 =0.
            endif               
        endif

        if(T < -2.8) then
            V = Lf
        else
            V = (0.36*(T+2.8) + Lf)                 !*Eq.22*!
        endif

        if (V<=0.0) V=0.0 
        D = D0 + 0.5*V
        
    else
        D = -999.
    endif

    return
end subroutine



!***** Subroutine for ISI   *****!
subroutine ISIcalc(F,W, R)
implicit none
    real(4),intent(in) :: F, W
    real(4),intent(out):: R
    
    real(4) fW,m,fF
    
    if (F>=0. .AND. W>=0.) then
        m  =  (147.2*(101.0-F)) / (59.5+F)                  !*Eq.1*!
        fW  =  exp(0.05039*W)                               !*Eq.24*!
        fF  =  91.9*exp(-0.1386*m) * (1.+(m**5.31)/4.93e7)  !*Eq.25*!
        R =  0.208 * fW * fF                                !*Eq.26*!
    else
        R = -999.
    endif

    return
end subroutine


!***** Subroutine for BUI  *****!
subroutine BUIcalc(P,D,U)
implicit none
    real(4), intent(in) :: P,D
    real(4), intent(out):: U

    if (P>=0. .AND. D>=0.) then
        if (P <= 0.4*D) then
            U=(0.8*P*D) / (P+0.4*D)                         !*Eq.27a*!
        else
            U=P-(1.-0.8*D/(P+0.4*D))*(0.92+(0.0114*P)**1.7) !*Eq.27b*!
        endif
        if (U<0.0) U=0.0
    else
        U=-999.
    endif

    return
end subroutine



!***** Subroutine for FWI *****!
subroutine FWIcalc(R,U,S)
implicit none
    real(4), intent(in) :: R,U
    real(4), intent(out):: S
    
    real(4) fD,B

    if (R>=0. .AND. U>=0.) then
        if (U<=80.0) then
            fD=0.626*U**0.809 + 2.                  !*Eq.28a*!
        else
            fD = 1000./(25.+108.64*exp(-0.023*U))   !*Eq.28b*!
        endif  
        B = 0.1*R*fD                                !*Eq.29*!
        if(B>1.) then
            S=exp(2.72*(0.434*log(B))**0.647)       !*Eq.30a*!
        else
            S=B                                     !*Eq.30b*!
        endif
    else
        S=-999.
    endif

    return
end subroutine

!***** Subroutine for FWI *****!
subroutine DSRcalc(FWI, DSR)
implicit none
    real(4), intent(in) :: FWI
    real(4), intent(out):: DSR

    if (FWI>=0.) then
        DSR = 0.0272*(FWI**1.77)               ! KRA - 2019-09-12
    else
        DSR = -999.
    endif

    return
end subroutine

end module fwiindices !!!***** END OF MODULE *****!!!



!****************************************!
!        Main program                    !
!****************************************!
program  fwicalculation
use fwiindices
implicit none

    real(4)     ::  temp,rhum,prcp,wind                        ! four weather variables
    real(4)     ::  ffmc, dmc, dc                              ! FFMC, DMC, and DC to be calculated 
    real(4)     ::  isi, bui, fwi, dsr                         ! ISI, BUI, FWI, and DSR to be calculated 
    real(4)     ::  ffmcx, dmcx, dcx                           ! FFMC, DMC, and DC to be calculated 
    real(4)     ::  isix, buix, fwix, dsrx                     ! ISI, BUI, FWI, and DSR to be calculated     
    real(4)     ::  ffmc0, dmc0, dc0, fwi0                     ! yesterday's FFMC, DMC, and DC values
    real(4)     ::  Lat, lon, elev, lats(3000)
    integer  ::  wmo, mth, day, pday, calcstatus                     ! month and day for each observation
    integer  ::  iii, i, io, imax, pwmo , tz_correct, wmos(3000)                                    ! utility integer

    ffmc0 = 85.0                                            ! First initialize
    dmc0  =  6.0                                            ! FFMC, DMC, and DC
    dc0   = 15.0                                            ! here
    
    ffmc0 = 71.8                                            ! Jack's values - 2019-09-12
    dmc0  =  5.2                                            ! Jack's values - 2019-09-12
    dc0   = 42.2                                            ! Jack's values - 2019-09-12

    ffmc0 = -999.
    dmc0  = -999.
    dc0   = -999.
    fwi0  = -999.
    pwmo  = -999
    pday = 0

    RefLat = 92;
    Lat = 45.42472222; ! Ottawa
    
    Lat = 53.57 ! 71002

!    RefLat = 46;
    

!    open(unit=1, file='data.txt', status='old')             ! open data file
    open(unit=1, file='fwi2018.txt', status='old')             ! open data file
    open(unit=2, file='fwioutput.txt',status='new')         ! output file
    open(unit=3, file='stns.txt', status='old')             ! open data file

    i=1
    do
        read(3, *, iostat=io) wmos(i), lon, lats(i), elev, tz_correct
        if(io/=0) exit                                  ! reach the end of file        
        i = i+1
    end do

    imax=i
    iii=0

    do
! wmo   month   day   temp   rh   ws   precip   ffmc   dmc   dc   bui   isi   fwi   dsr   calcstatus

        read(1,*,iostat=io) wmo,mth,day,temp,rhum,wind,prcp,&
        ffmc,dmc,dc,bui,isi,fwi,dsr,calcstatus          ! read in data day by day 
        JD = days(mth) + day
        
!        if (pwmo>0 .AND. wmo/=pwmo) exit

        if (pwmo>0 .AND. wmo/=pwmo) then
            do i=1, imax
                if (wmo == wmos(i) ) then
                    Lat = lats(i)
                    exit
                end if
            end do
        end if

        if(io/=0) exit                                  ! reach the end of file
        
        if (day == 1 .OR. day == pday+1) then            ! a means to skip missing days
            if (ffmc0>0 .AND. dmc0>0 .AND. dc0>0 .AND. &
                ffmc>0 .AND. dmc>0 .AND. dc>0 .AND. &
                temp >-10 .AND. rhum > 0 .AND. wind >= 0) then
                iii = iii +1
                if(rhum>=100.0) rhum=100.0                      ! make sure RHUM <= 100.0
                
                call FFMCcalc(temp,rhum,wind,prcp,ffmc0,ffmcx)   ! calculate FFMC
                call DMCcalc(temp,rhum,prcp,dmc0,mth,JD,Lat,RefLat,dmcx)      ! calcualte DMC
                call DCcalc(temp,prcp,dc0,mth,Lat,dcx)             ! calcualte DC
                if (fwi0 < 0) dcx = dc0                          ! a quirk about CWFIS: it carries over day 1 DC tro day 2

                call ISIcalc (ffmcx,wind,isix)                    ! calcualte ISI
                call BUIcalc (dmcx,dcx,buix)                       ! calculate BLL
                call FWIcalc (isix,buix,fwix)                      ! calculate FWI
                call DSRcalc (fwix,dsrx)                          ! calculate DSR
                
                ffmcx = INT(10.*ffmcx+.5)/10.                        ! KRA - 2019-09-12
                dmcx = INT(10.*dmcx+.5)/10.                          ! KRA - 2019-09-12
                dcx = INT(10.*dcx+.5)/10.                            ! KRA - 2019-09-12                

                isix = INT(10.*isix+.5)/10.                        ! KRA - 2019-09-12
                buix = INT(10.*buix+.5)/10.                          ! KRA - 2019-09-12
                fwix = INT(10.*fwix+.5)/10.                            ! KRA - 2019-09-12                
                dsrx = INT(10.*dsrx+.5)/10.                            ! KRA - 2019-09-12                

                if ((ffmc/=ffmcx .OR. dmc/=dmcx .OR. dc/=dcx) .AND. fwi0>0) then
                    write(6,20) wmo,mth,day,ffmc,ffmcx,dmc,dmcx,dc,dcx
    20              FORMAT(i8, 2i4, 6f8.1)
    
!                if ((isi/=isix .OR. bui/=buix .OR. fwi/=fwix .OR. dsr/=dsrx) .AND. &
!                if ((isi/=isix ) .AND. &
!                    fwi>=0 .AND. bui>=0 .AND. isi>=0 .AND. fwix>=0 .AND. buix>=0 .AND. isix>=0) then
!                    write(6,30) wmo,mth,day,isi,isix,bui,buix,fwi,fwix,dsr,dsrx

    30              FORMAT(i8, 2i4, 8f8.1)
                end if
                
            end if
        end if

        ffmc0 = ffmc                                    ! use today's values
        dmc0  = dmc                                     ! for tomorrow's
        dc0   = dc                                      ! calculation
        fwi0  = fwi                                      ! calculation
        pwmo  = wmo
        pday  = day

        write(2,10) mth,day,temp,rhum,wind,prcp,&       ! output results
            ffmc, dmc, dc, isi, bui, fwi, dsr            ! to scereen (or to file) ! KRA - 2019-09-12

          
!        ffmc0 = INT(10.*ffmc)/10.                        ! KRA - 2019-09-12
!        dmc0 = INT(10.*dmc)/10.                          ! KRA - 2019-09-12
!        dc0 = INT(10.*dc)/10.                            ! KRA - 2019-09-12

!        write(*,*) ffmc, dmc, dc, ffmc0, dmc0, dc0
 
!       write(6,10) mth,day,temp,rhum,wind,prcp,&       ! output results
!            ffmc, dmc, dc, isi, bui, fwi, dsr            ! to scereen (or to file) ! KRA - 2019-09-12
 
 
10        format(i4,i5,4f6.1,6f8.1,f9.2)                  ! output data format
    enddo
    
    write(*,*) iii
    close(1)       
    close(2)  
end program fwicalculation
