/*----------------------------------------------------------------------
  Program..: GEO.C
  Author...: KERRY ANDERSON, programmer extraordinaire
  Date.....: Dec 9, 1999
  Notice...: Copyright (c) 1999, Forestry Canada, All Rights Reserved
  Notes....: Converts point coords to appropriate map projection coords
  Reserved.:
----------------------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include "geo.h"

/********************************************************************************
   Subroutine Albers: translates between lat/longs and Albers coordinates

   if x and y are zero, then calculates from lat/long to Albers
   if not, then calculates from Albers to lat/long

   equations taken from

Snyder, J.P.  1987.  Map Projections - A Working Manual.  U.S. Geol. Survey
     Professional Paper 1395.  U.S. Govt. Printing Office, Waskington.  383 pp.
********************************************************************************/
void Albers (double *x, double *y,
              double *lat, double *lng, double *cm, double *th1, double *th2)

{
double pi = 3.1415926536;
double a,e,phi,phi0,phi1,phi2,lamda,lamda0,
       m1,m2,n,rho,rho0,theta;
double esinphi0, esinphi1, esinphi2, eover2, esinphi, piover4;
double q, q0, q1, q2, C, dphi;
double piover180;

/* radius and ellipsoidal of the Earth */
   a = 6378206.4;
   e = .0822719;

   piover180 = 0.0174532925199;

/* standard parallels */
   phi1 = *th1 * piover180; /* pi / 180 */
   phi2 = *th2 * piover180;

/* lat/lng of the origin of the rectangular coordinates (the North Pole) */
   phi0 = 1.5707963268; /* pi / 2 */
   lamda0 = *cm * piover180;

   esinphi0 = e * sin(phi0);
   esinphi1 = e * sin(phi1);
   esinphi2 = e * sin(phi2);
   eover2 = 0.04113595;  /* e / 2 */
   piover4 = 0.785398163398;

   m1 = cos(phi1) / sqrt(1 - esinphi1 * esinphi1);                 /* 14-15 */
   m2 = cos(phi2) / sqrt(1 - esinphi2 * esinphi2);                 /* 14-15 */

   if (e==0)
   {
	   q0 = 2*sin(phi0);											/* 3-12 */
	   q1 = 2*sin(phi1);											/* 3-12 */
	   q2 = 2*sin(phi2);											/* 3-12 */
   }
   else
   {
      q0 = (1 - e*e)*(sin(phi0)/(1 - esinphi0 * esinphi0) -
	        1/(2*e) * log((1 - esinphi0)/(1 + esinphi0)));			/* 3-12 */
      q1 = (1 - e*e)*(sin(phi1)/(1 - esinphi1 * esinphi1) -
	        1/(2*e) * log((1 - esinphi1)/(1 + esinphi1)));			/* 3-12 */
      q2 = (1 - e*e)*(sin(phi2)/(1 - esinphi2 * esinphi2) -
	        1/(2*e) * log((1 - esinphi2)/(1 + esinphi2)));			/* 3-12 */
   }

   if (phi1==phi2)
      n = sin(phi1);
   else
	  n = (m1*m1 - m2*m2)/(q2 - q1);								/* 14-14 */

   C = m1*m1 + n*q1;												/* 14-13 */
   rho0 = a*sqrt(C - n*q0)/n;										/* 14-12a */

   if (*x == 0. && *y == 0.)
   {

/* Calculating *x, *y from *lat, *lng */
      phi = *lat * piover180;
      lamda = *lng * piover180;
      esinphi = e * sin(phi);

      if (e==0)
         q = 2*sin(phi);												/* 3-12 */
      else
         q = (1 - e*e)*(sin(phi)/(1 - esinphi * esinphi) -
	          1/(2*e) * log((1 - esinphi)/(1 + esinphi)));				/* 3-12 */

      rho = a*sqrt(C - n*q)/n;											/* 14-12a */
      theta = n*(lamda - lamda0);										/* 14-4 */
      *x = rho * sin(theta);											/* 14-1 */
      *y = rho0 - rho*cos(theta);										/* 14-2 */
   }
   else
   {
      rho = sqrt( (*x)*(*x) + (rho0 - (*y))*(rho0 - (*y)));				/* 14-10 */
	  if (n >= 0)
	     theta = atan2((*x), rho0-(*y));								/* 14-11 */
	  else
	     theta = atan2(-(*x), -rho0+(*y));								/* 14-11 */

      q = (C-rho*rho*n*n/a/a)/n;										/* 14-19 */

	  if (q == 1 - (1-e*e)/(2*e) * log((1-e)/(1+e)) )
		 phi = pi/2;
	  else if (q == -(1 - (1-e*e)/(2*e) * log((1-e)/(1+e))) )
		 phi = -pi/2;
	  else
         phi = asin(q/2);
	     dphi = phi;
	     while (dphi/phi > .00001)
		 {
	        esinphi = e*sin(phi);
	        dphi = (1-esinphi*esinphi)*(1-esinphi*esinphi)/(2*cos(phi))
		                                 *
				  ( q/(1-e*e) - sin(phi)/(1-esinphi*esinphi)
		          + 1/(2*e)*log((1-esinphi)/(1+esinphi)) );				/* 3-16 */
			phi = phi + dphi;
		 }

	  lamda = lamda0+theta/n;											/* 14-9 */

      *lat = phi * 57.2957795131;  /* 180 / pi */
      *lng = lamda * 57.2957795131; /* 180 / pi */
   }
}

/********************************************************************************
   Subroutine Lambert: translates between lat/longs and Lambert coordinates

   if x and y are zero, then calculates from lat/long to Lambert
   if not, then calculates from Lambert to lat/long

   equations taken from

Snyder, J.P.  1987.  Map Projections - A Working Manual.  U.S. Geol. Survey
     Professional Paper 1395.  U.S. Govt. Printing Office, Waskington.  383 pp.
********************************************************************************/
void Lambert (double *x, double *y,
              double *lat, double *lng, double *cm, double *th1, double *th2)

{
double pi = 3.1415926536;
double a,e,phi,phi0,phi1,phi2,lamda,lamda0,
       k,m,m1,m2,t,t0,t1,t2,n,F,rho,rho0,theta,chi;
double esinphi1, esinphi2, eover2, esinphi, piover4;
double eto2, eto4, eto6, eto8, piover180;

/* radius and ellipsoidal of the Earth */
   a = 6378206.4;
   e = .0822719;

   piover180 = 0.0174532925199;

/* standard parallels */
   phi1 = *th1 * piover180; /* pi / 180 */
   phi2 = *th2 * piover180;

/* lat/lng of the origin of the rectangular coordinates (the North Pole) */
   phi0 = 1.5707963268; /* pi / 2 */
   lamda0 = *cm * piover180;

   esinphi1 = e * sin(phi1);
   esinphi2 = e * sin(phi2);
   eover2 = 0.04113595;  /* e / 2 */
   piover4 = 0.785398163398;
   m1 = cos(phi1) / sqrt(1 - esinphi1 * esinphi1);                 /* 14-15 */
   m2 = cos(phi2) / sqrt(1 - esinphi2 * esinphi2);                 /* 14-15 */
   t1 = tan(piover4 - phi1 / 2) /
        pow ((1 - esinphi1)/(1 + esinphi1),eover2);          /* 15-9 */
   t2 = tan(piover4 - phi2 / 2) /
        pow ((1 - esinphi2)/(1 + esinphi2),eover2);          /* 15-9 */
   t0 = 0.;
   n = (log(m1) - log(m2)) / (log(t1) - log(t2));                   /* 15-8 */
   F = m1 / (n * pow(t1,n));                                        /* 15-10 */
   rho0 = a * F * pow(t0,n);                                        /* 15-7a */

   if (*x == 0. && *y == 0.)
      {

/* Calculating *x, *y from *lat, *lng */
      phi = *lat * piover180;
      lamda = *lng * piover180;
      esinphi = e * sin(phi);


      t = tan(piover4 - phi / 2) /
          pow((1 - esinphi)/(1 + esinphi),eover2);           /* 15-9 */
      rho = a * F * pow(t,n);                                       /* 15-7 */
      if (lamda - lamda0 > 180.)
          theta = n * (lamda - lamda0 - 360);                       /* 14-4 */
      else if (lamda - lamda0 < -180.)
          theta = n * (lamda - lamda0 + 360);                       /* 14-4 */
      else
          theta = n * (lamda - lamda0);                             /* 14-4 */
      *x = rho * sin(theta);                                        /* 14-1 */
      *y = rho0 - rho * cos(theta);                                 /* 14-2 */
      m = cos(phi) / sqrt(1 - esinphi * esinphi);                 /* 14-15 */
      k = rho * n/a/m;                                              /* 14-16 */
      }
   else
      {

/* Calculating *lat, *lng from *x, *y */
      if (n >= 0.)
         rho = sqrt(*x * *x + (rho0 - *y) * (rho0 - *y));                  /* 14-10 */
      else
         rho = -sqrt(*x * *x + (rho0 - *y) * (rho0 - *y));                 /* 14-10 */

      if (n >= 0.)
         theta = atan(*x / (rho0 - *y));                            /* 14-11 */
      else
         theta = atan(-*x / (-rho0 + *y));                          /* 14-11 */

      t = pow(rho/a/F,1/n);                                         /* 15-11 */
      chi = 1.5707963268 - 2 * atan(t);                                   /* 7-13 */
      eto2 = 0.00676866552961;
      eto4 = 0.0000458148330517;
      eto6 = 0.000000310105281222;
      eto8 = 0.00000000209899892756;
      phi = chi
    + (eto2/2 + 5*eto4/24 + eto6/12 + 13*eto8/360) * sin(2*chi)
    + (7*eto4/48 + 29*eto6/240 + 811*eto8/11520) * sin(4*chi)
    + (7*eto6/120 + 81*eto8/1120) * sin(6*chi)
    + (4279*eto8/161280) * sin(8*chi);                          /* 3-5 */
      lamda = theta / n + lamda0;

      *lat = phi * 57.2957795131;  /* 180 / pi */
      *lng = lamda * 57.2957795131; /* 180 / pi */
      }
}

/********************************************************************************
   Subroutine Polar: translates between lat/longs and
   Polar Sterographic coordinates

   if x and y are zero, then calculates from lat/long to Polar
   if not, then calculates from Polar to lat/long

   equations taken from

Snyder, J.P.  1987.  Map Projections - A Working Manual.  U.S. Geol. Survey
     Professional Paper 1395.  U.S. Govt. Printing Office, Waskington.  383 pp.
********************************************************************************/
void Polar (double *x,
            double *y,
            double *lat,
            double *lng,
            double lat0,
            double lng0,
            double latc)
{
double e2,e4,e6,e8,phi,phi1,phic,lamda,lamda0,A,
       ce,k,m,m1,mc,t,tc,rho,chi,chi1;

/* radius and ellipsoidal of the Earth */
double PI = 3.1415926536,
       a = 6378206.4,
       e = .0822719,
       k0 = 1.0;

   e2 = e*e;
   e4 = e2*e2;
   e6 = e2*e4;
   e8 = e4*e4;

/* convert coordinates to radians */
   phic = latc*PI/180;
   phi1 = lat0*PI/180;
   lamda0 = lng0*PI/180;
   phi = *lat*PI/180;
   lamda = *lng*PI/180;

/* These routines presently do not account for south polar aspects */

   if (*x == 0. && *y == 0.)   /* Calculating *x, *y from *lat, *lng */
      {
      if (lat0 == 90.)
         {
         t = sqrt((1-sin(phi))/(1+sin(phi))*
             pow((1+e*sin(phi))/(1-e*sin(phi)),e));                  /* 15-9 */

         if (latc == 90.)
            rho = 2*a*k0*t/sqrt(pow(1+e,1+e)*pow(1-e,1-e));          /* 21-33 */
         else
            {
            tc = sqrt((1-sin(phic))/(1+sin(phic))*
                pow((1+e*sin(phic))/(1-e*sin(phic)),e));             /* 15-9 */

            mc = cos(phic)/sqrt(1-e*e*sin(phic)*sin(phic));          /* 14-15 */
            rho = a*mc*t/tc;                                         /* 21-34 */
            }
         *x = rho*sin(lamda - lamda0);                               /* 21-30 */
         *y = -rho*cos(lamda - lamda0);                              /* 21-31 */
         k = rho/(a*m);	                                             /* 21-32 */
         }
      else
         {
         chi = 2*atan(sqrt((1+sin(phi))/(1-sin(phi))*
                 pow((1-e*sin(phi))/(1+e*sin(phi)),e)))-PI/2;        /* 3-1a */

         chi1 = 2*atan(sqrt((1+sin(phi1))/(1-sin(phi1))*
                 pow((1-e*sin(phi1))/(1+e*sin(phi1)),e)))-PI/2;      /* 3-1a */

         m = cos(phi)/sqrt(1-e*e*sin(phi)*sin(phi));                 /* 14-15 */
         m1 = cos(phi1)/sqrt(1-e*e*sin(phi1)*sin(phi1));             /* 14-15 */

         A = 2*a*k0*m1/(cos(chi1)*(1+sin(chi1)*sin(chi)
             + cos(chi1)*cos(chi)*cos(lamda-lamda0)));               /* 21-27 */

         *x = A*cos(chi)*sin(lamda-lamda0);                          /* 21-24 */
         *y = A*(cos(chi1)*sin(chi) -
             sin(chi1)*cos(chi)*cos(lamda-lamda0));                  /* 21-25 */
         k = A*cos(chi/(a*m));                                       /* 21-26 */
         }
      }
   else  /* Calculating *lat, *lng from *x, *y */
      {
      if (lat0 == 90.)
         {
         rho = sqrt(*x * *x + *y * *y);                              /* 20-18 */

         if (latc == 90)
            t = rho*sqrt(pow(1+e,1+e)*pow(1-e,1-e))/(2*a*k0);        /* 21-39 */
         else
            {
            t = sqrt((1-sin(phic))/(1+sin(phic))*
                pow((1+e*sin(phic))/(1-e*sin(phic)),e));             /* 15-9 */
            mc = cos(phic)/sqrt(1-e*e*sin(phic)*sin(phic));          /* 14-15 */
            t = rho*tc/(a*mc);                                       /* 21-40 */
            }
         chi = PI/2 -2*atan(t);                                      /* 7-13 */

         phi = chi + (e2/2 + 5*e4/24 + e6/12 +13*e8/360)*sin(2*chi)
             + (7*e4/48 + 29*e6/240 + 811*e8/11520)*sin(4*chi)
             + (7*e6/120 + 81*e8/1120)*sin(6*chi)
             + (4279*e8/161280)*sin(8*chi);                          /* 3-5 */

         lamda = lamda0 + atan2(*x,-(*y));                           /* 20-16 */
         if (phi > PI/2)
            {
            phi = PI - phi;
            lamda = lamda - PI;
            }
         }
      else
         {

         m1 = cos(phi1)/sqrt(1-e*e*sin(phi1)*sin(phi1));             /* 14-15 */

         chi1 = 2*atan(sqrt((1+sin(phi1))/(1-sin(phi1))*
                 pow((1-e*sin(phi1))/(1+e*sin(phi1)),e)))-PI/2;      /* 3-1a */

         rho = sqrt(*x * *x + *y * *y);                              /* 20-18 */
         ce = 2*atan2(rho*cos(chi1),(2*a*k0*m1));                    /* 21-38 */

         chi = asin(cos(ce)*sin(chi1) + (*y*sin(ce)*cos(chi1)/rho)); /* 21-37 */

         phi = chi + (e2/2 + 5*e4/24 + e6/12 +13*e8/360)*sin(2*chi)
             + (7*e4/48 + 29*e6/240 + 811*e8/11520)*sin(4*chi)
             + (7*e6/120 + 81*e8/1120)*sin(6*chi)
             + (4279*e8/161280)*sin(8*chi);                          /* 3-5 */

         lamda = lamda0 + atan2(*x*sin(ce),
                 rho*cos(chi1)*cos(ce)- *y*sin(chi1)*sin(ce));       /* 21-26 */
         }
      *lat = phi*180/PI;
      *lng = lamda*180/PI;
      }
}

/********************************************************************************/
/* subroutine utm: translates between lat/longs and UTM coordinates

   if zone > 0, then lat/long values are calculated for a given UTM
   if zone = 0, then UTM values are calculated for a given lat/long
   if zone < 0, then UTM values are calculated for a given lat/long
                in a given zone (indicated by the negative zone value)

   when converting utm to lat/long
   if hemisphere is => 0 then utm coordinates are for the northern hemisphere
   if hemisphere is < 0  then utm coordinates are for the southern hemisphere

   equations are taken from

Snyder, J.P.  1987.  Map Projections - A Working Manual.  U.S. Geol. Survey
     Professional Paper 1395.  U.S. Govt. Printing Office, Waskington.  383 pp.

   using the Clarke 1866 ellispoid.  */

void utm(double *dlat,
         double *dlong,
         double *easting,
         double *northing,
         int    *zone,
         int    *hemisphere)
{
double a,e2,ko,Mo, phi,phi1,lamda,lamdao,eP2,N,T,C,A,M,A2,A3,A4,A5,A6,
       x,y,k,C1,T1,N1,R1,D,mu,e1;
int z;

   a = 6378206.4;
   e2 = 0.00676866;
   ko = 0.9996;
   Mo = 0.;
   eP2 = e2/(1-e2);                                               /* 8-12 */

   if(*zone <= 0)
   {
      phi = *dlat*3.1415926/180;
      lamda = *dlong*3.1415926/180;
      if(*zone == 0)
         z = (int)((186+*dlong)/6);
      else
         z = -(*zone);
      lamdao = (z*6 - 183)*3.1415926/180;

      N = a/sqrt(1 - e2*sin(phi)*sin(phi));                          /* 4-20 */
      T = tan(phi)*tan(phi);                                         /* 8-13 */
      C = eP2*cos(phi)*cos(phi);                                     /* 8-14 */
      A = (lamda - lamdao)*cos(phi);                                 /* 8-15 */
      M = 111132.0894*(*dlat) - 16216.94*sin(2*phi) + 17.21*sin(4*phi)
        - 0.02*sin(6*phi);                                           /* 3-22 */

      A2 = A*A;
      A3 = A2*A;
      A4 = A2*A2;
      A5 = A2*A3;
      A6 = A3*A3;

      x = ko*N*(A + (1 - T + C)*A3/6
        + (5 - 18*T + T*T + 72*C - 58*eP2)*A5/120);                  /* 8-9 */
      y = ko*(M - Mo + N*tan(phi)*(A2/2 + (5 - T + 9*C + 4*C*C)*A4/24
        + (61 - 58*T + T*T + 600*C - 330*eP2)*A6/720));              /* 8-10 */
      k = ko*(1 + (1 + C)*A2/2 + (5 - 4*T + 42*C + 13*C*C - 28*eP2)*A4/24
        + (61 - 148*T + 16*T*T)*A6/720);                             /* 8-11 */

      if (*dlat >= 0.)
         *northing = y;
      else
         *northing = y + 10000000;
      *easting = x + 500000;
      *zone = z;
   }
   else
   {
      if (*hemisphere < 0)
         y = *northing - 10000000;
      else
         y = *northing;
      x = *easting - 500000;
      z = *zone;
      lamdao = (z*6 - 183)*3.1415926/180;


      M = Mo + y/ko;                                                 /* 8-20 */
      e1 = (1 - sqrt(1 - e2))/(1 + sqrt(1 - e2));                    /* 3-24 */
      mu = M/(a*(1. - e2/4. - 3.*e2*e2/64. - 5.*e2*e2*e2/256.));     /* 7-19 */

      phi1 = mu + (525.3295*sin(2.*mu)+0.7805*sin(4.*mu)
           + 0.0016*sin(6.*mu))*3.1415926/180./3600.;                /* 3-27 */

      C1 = eP2*cos(phi1)*cos(phi1);                                  /* 8-21 */
      T1 = tan(phi1)*tan(phi1);                                      /* 8-22 */
      N1 = a/sqrt(1 - e2*sin(phi1)*sin(phi1));                       /* 4-20 */
      R1 = a*(1-e2)/pow((1-e2*sin(phi1)*sin(phi1)),1.5);             /* 8-24 */
      D = x/(N1*ko);                                                 /* 8-25 */


      phi = phi1
     - (N1*tan(phi1)/R1)*(D*D/2-(5 + 3*T1 + 10*C1 - 4*C1*C1 - 9*eP2)*D*D*D*D/24
     + (61 + 90*T1 + 298*C1 + 45*T1*T1 - 252*eP2 - 3*C1*C1)*D*D*D*D*D*D/720);
                                                                     /* 8-17 */

      lamda = lamdao + (D - (1 + 2*T1 + C1)*D*D*D/6
          + (5 - 2*C1 + 28*T1 - 3*C1*C1 + 8*eP2 + 24*T1*T1)*D*D*D*D*D/120)/cos(phi1);
                                                                     /* 8-18 */
      *dlat = phi*180/3.1415926;
      *dlong = lamda*180/3.1415926;

  }
}
